/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFAD_IM_H__
#define __BFAD_IM_H__

#include "fcs/bfa_fcs_fcpim.h"
#include "bfad_im_compat.h"

#define FCPI_NAME " fcpim"

#ifndef KOBJ_NAME_LEN
#define KOBJ_NAME_LEN           20
#endif

void bfad_flags_set(struct bfad_s *bfad, uint32_t flags);
bfa_status_t bfad_im_module_init(void);
void bfad_im_module_exit(void);
bfa_status_t bfad_im_probe(struct bfad_s *bfad);
void bfad_im_probe_undo(struct bfad_s *bfad);
void bfad_im_probe_post(struct bfad_im_s *im);
bfa_status_t bfad_im_port_new(struct bfad_s *bfad, struct bfad_port_s *port);
void bfad_im_port_delete(struct bfad_s *bfad, struct bfad_port_s *port);
void bfad_im_port_online(struct bfad_s *bfad, struct bfad_port_s *port);
void bfad_im_port_offline(struct bfad_s *bfad, struct bfad_port_s *port);
void bfad_im_port_clean(struct bfad_im_port_s *im_port);
int  bfad_im_scsi_host_alloc(struct bfad_s *bfad,
				struct bfad_im_port_s *im_port);
void bfad_im_scsi_host_free(struct bfad_s *bfad,
				struct bfad_im_port_s *im_port);

#define MAX_FCP_TARGET 1024
#define MAX_FCP_LUN 512
#define BFAD_TARGET_RESET_TMO 60
#define BFAD_LUN_RESET_TMO 60
#define ScsiResult(host_code, scsi_code) (((host_code) << 16) | scsi_code)
#define BFA_QUEUE_FULL_RAMP_UP_TIME 120

/*
 * itnim flags
 */
#define ITNIM_MAPPED		0x00000001

#define SCSI_TASK_MGMT		0x00000001
#define IO_DONE_BIT			0

struct bfad_itnim_data_s {
	struct bfad_itnim_s *itnim;
};

struct bfad_im_port_s {
	struct bfad_s         *bfad;
	struct bfad_port_s    *port;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 9)
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
	struct work_struct scsi_scan_work;
#else
	struct tq_struct scsi_scan_work;
#endif
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
	struct work_struct port_delete_work;
#else
	struct tq_struct port_delete_work;
#endif
	int             idr_id;
	uint16_t        cur_scsi_id;
	struct list_head binding_list;
	struct Scsi_Host *shost;
#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 9)
	struct bfad_itnim_data_s itnim_data_table[MAX_FCP_TARGET];
#endif
	struct list_head itnim_mapped_list;
};

enum bfad_itnim_state {
	ITNIM_STATE_NONE,
	ITNIM_STATE_ONLINE,
	ITNIM_STATE_OFFLINE_PENDING,
	ITNIM_STATE_OFFLINE,
	ITNIM_STATE_TIMEOUT,
	ITNIM_STATE_FREE,
};

/*
 * Per itnim data structure
 */
struct bfad_itnim_s {
	struct list_head list_entry;
	bfa_fcs_itnim_t fcs_itnim;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
	struct work_struct itnim_work;
#else
	struct tq_struct itnim_work;
#endif
	uint32_t        flags;
	enum bfad_itnim_state state;
	struct bfad_im_s *im;
	struct bfad_im_port_s *im_port;
	struct bfad_rport_s *drv_rport;
	struct fc_rport *fc_rport;
	bfa_itnim_t *bfa_itnim;
	uint16_t        scsi_tgt_id;
	uint16_t        queue_work;
	unsigned long	last_ramp_up_time;
	unsigned long	last_queue_full_time;
};

enum bfad_binding_type {
	FCP_PWWN_BINDING = 0x1,
	FCP_NWWN_BINDING = 0x2,
	FCP_FCID_BINDING = 0x3,
};

struct bfad_fcp_binding {
	struct list_head list_entry;
	enum bfad_binding_type binding_type;
	uint16_t        scsi_target_id;
	uint32_t        fc_id;
	wwn_t           nwwn;
	wwn_t           pwwn;
};

struct bfad_im_s {
	struct bfad_s         *bfad;
	struct workqueue_struct *drv_workq;
	char            drv_workq_name[KOBJ_NAME_LEN];
};

void bfad_os_spin_os_lock_irq(struct Scsi_Host *);
void bfad_os_spin_os_unlock_irq(struct Scsi_Host *);
void bfad_os_fc_release_transp(struct Scsi_Host *shost);
struct Scsi_Host *bfad_os_scsi_host_alloc(struct bfad_im_port_s *im_port,
				struct bfad_s *);
int bfad_os_scsi_register(struct Scsi_Host *shost,
				struct bfad_im_port_s *im_port, void *key);
int bfad_os_fc_attach(struct Scsi_Host *shost,
				struct fc_function_template *fct);
int bfad_os_reset_tgt(struct scsi_cmnd *cmnd);
void bfad_os_scsi_set_pci_device(struct Scsi_Host *shost,
				struct pci_dev *pcidev);
void bfad_os_select_queue_depths(struct bfad_im_port_s *im_port);
bfa_status_t bfad_os_thread_workq(struct bfad_s *bfad);
void bfad_os_destroy_workq(struct bfad_im_s *im);
void bfad_os_queue_work(void *workq, void *work);
void bfad_os_itnim_alloc(struct bfad_itnim_s *itnim_drv);
void bfad_os_itnim_process(struct bfad_itnim_s *itnim_drv);
void bfad_os_itnim_tov(struct bfad_itnim_s *itnim_drv);
void bfad_os_scsi_unregister(struct Scsi_Host *shost);
void bfad_os_scsi_remove_host(struct Scsi_Host *shost);
void bfad_os_scsi_put_host(struct Scsi_Host *shost);
int bfad_os_scsi_add_host(struct Scsi_Host *shost,
		struct bfad_im_port_s *im_port, struct bfad_s *bfad);
#if defined(__VMKERNEL_MODULE__) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
int bfad_os_fc_attach(struct Scsi_Host *shost,
				  struct fc_function_template *fct);
#else
#define bfad_os_fc_attach(x, y)	0
#endif

struct bfad_itnim_data_s *bfad_os_itnim_data(struct bfad_im_port_s *im_port,
				struct scsi_cmnd *cmnd);
void bfad_os_fc_host_init(struct bfad_im_port_s *im_port);
void bfad_os_fc_remove_host(struct Scsi_Host *shost);
void bfad_os_init_work(struct bfad_im_port_s *im_port);
int  bfad_os_dev_init(struct Scsi_Host *shost);
#if defined(__VMKERNEL_MODULE__) || (LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0))
void bfad_os_transp_templ(struct Scsi_Host *sh,
				struct scsi_transport_template *stt);
dma_addr_t bfad_os_dma_map_single(struct device *dev, void *cpu_addr,
			size_t size, enum dma_data_direction direction);
void bfad_os_dma_unmap_single(struct device *dev, dma_addr_t dma_addr,
			size_t size, enum dma_data_direction direction);
int bfad_os_dma_map_sg(struct device *dev, struct scatterlist *sg, int nents,
			enum dma_data_direction direction);
void bfad_os_dma_unmap_sg(struct device *dev, struct scatterlist *sg,
			int nhwentries, enum dma_data_direction direction);
#else
dma_addr_t bfad_os_dma_map_single(struct pci_dev *dev, void *cpu_addr,
                        size_t size,
                        enum dma_data_direction direction);
void bfad_os_dma_unmap_single(struct pci_dev *dev,
                        dma_addr_t dma_addr, size_t size,
                        enum dma_data_direction direction);
int bfad_os_dma_map_sg(struct pci_dev *dev, struct scatterlist *sg,
                        int nents,
                        enum dma_data_direction direction);
void bfad_os_dma_unmap_sg(struct pci_dev *dev, struct scatterlist *sg,
                        int nhwentries,
                        enum dma_data_direction direction);
#endif
int bfad_os_im_dma_map(struct bfad_s *bfad, struct scsi_cmnd *cmnd);
void bfad_os_im_dma_unmap(struct bfad_s *bfad, struct scsi_cmnd *cmnd);
int bfad_os_idr_get_new(struct idr *idp, void *ptr, int *id);
void bfad_os_scsi_state_changed(struct Scsi_Host *shost, bfad_itnim_t *itnim);
void bfad_os_scsi_scan(struct bfad_im_port_s *im_port);
void bfad_os_scsi_host_free(struct bfad_s *bfad,
				 struct bfad_im_port_s *im_port);
void bfad_wwn_to_str(wwn_t wwn, char *str);
void bfad_fcid_to_str(uint32_t fcid, char *str);
void bfad_os_ramp_up_qdepth(struct bfad_itnim_s *itnim,
				 struct scsi_device *sdev);
void bfad_os_handle_qfull(struct bfad_itnim_s *itnim, struct scsi_device *sdev);
void bfad_os_set_dev_loss_tmo(struct scsi_device *sdev);
struct bfad_itnim_s *bfad_os_get_itnim(struct bfad_im_port_s *im_port, int id);
int bfad_os_itnim_online_check(struct bfad_itnim_s *intim);

/*
 * scsi_host_template entries
 */
int bfad_im_queuecommand(struct scsi_cmnd *cmnd,
					void (*done)(struct scsi_cmnd *));
const char *bfad_im_info(struct Scsi_Host *host);
int bfad_im_slave_alloc(struct scsi_device *sdev);
void bfad_im_slave_destroy(struct scsi_device *sdev);
int bfad_im_abort_handler(struct scsi_cmnd *cmnd);
int bfad_im_reset_lun_handler(struct scsi_cmnd *cmnd);
int bfad_im_reset_bus_handler(struct scsi_cmnd *cmnd);
void bfad_im_itnim_unmap(struct bfad_im_port_s  *im_port,
				bfad_itnim_t *itnim);

extern struct scsi_host_template bfad_im_scsi_host_template;
extern struct scsi_host_template bfad_im_vport_template;
extern struct fc_function_template bfad_im_fc_function_template;
extern struct scsi_transport_template *bfad_im_scsi_transport_template;

#endif
