/* 
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 * 
 * See LICENSE.bfa for copyright and licensing details.
 */
/**
 * @file bfad_im_wrapper.c Linux driver IM module.
 * Wrapper functions for Linux ver 2.6.*
 */
#include "bfad_drv.h"
#include "bfad_im.h"
#include "bfad_trcmod.h"

int
bfad_os_dma_map_sg(struct device *dev, struct scatterlist *sg, int nents,
		   enum dma_data_direction direction)
{
	return dma_map_sg(dev, sg, nents, direction);
}

dma_addr_t
bfad_os_dma_map_single(struct device *dev, void *cpu_addr, size_t size,
		       enum dma_data_direction direction)
{
	return dma_map_single(dev, cpu_addr, size, direction);
}

void
bfad_os_dma_unmap_sg(struct device *dev, struct scatterlist *sg, int nhwentries,
		     enum dma_data_direction direction)
{
	dma_unmap_sg(dev, sg, nhwentries, direction);
}

void
bfad_os_dma_unmap_single(struct device *dev, dma_addr_t dma_addr, size_t size,
			 enum dma_data_direction direction)
{
	dma_unmap_single(dev, dma_addr, size, direction);
}

void
bfad_os_queue_work(void *workq, void *work)
{
	queue_work((struct workqueue_struct *)workq,
		   (struct work_struct *)work);
}

void
bfad_os_scsi_put_host(struct Scsi_Host *shost)
{
	scsi_host_put(shost);
}

int
bfad_os_scsi_add_host(struct Scsi_Host *shost, struct bfad_im_port_s *im_port,
			struct bfad_s *bfad)
{
	struct device *dev;

	if (im_port->port->pvb_type == BFAD_PORT_PHYS_BASE)
		dev = &bfad->pcidev->dev;
	else
		dev = &bfad->pport.im_port->shost->shost_gendev;

	return scsi_add_host(shost, dev);
}

void
bfad_os_scsi_remove_host(struct Scsi_Host *shost)
{
	scsi_remove_host(shost);
}

int
bfad_os_dev_init(struct Scsi_Host *shost)
{
	return 0;
}

void
bfad_os_fc_release_transp(struct Scsi_Host *shost)
{
	return;
}

void
bfad_os_scsi_unregister(struct Scsi_Host *shost)
{
	return;
}

void
bfad_os_select_queue_depths(struct bfad_im_port_s *im_port)
{
	return;
}

void
bfad_os_scsi_set_pci_device(struct Scsi_Host *shost, struct pci_dev *pcidev)
{
	return;
}

void
bfad_os_idr_init(struct idr *idr)
{
	return;
}

void
bfad_os_idr_destroy(struct idr *idr)
{
	return;
}

int
bfad_os_idr_get_new(struct idr *idp, void *ptr, int *id)
{
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
	int             idr;

	idr = idr_get_new(idp, NULL);
	if (idr != -1)
		*id = idr;
	return (idr == -1);
#else
	return idr_get_new(idp, NULL, id);
#endif
}

int
#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
bfad_os_idr_pre_get(struct idr *idr, int gfp_mask)
#else
bfad_os_idr_pre_get(struct idr *idr, gfp_t gfp_mask)
#endif
{
	return idr_pre_get(idr, GFP_KERNEL);
}

void
bfad_os_idr_remove(struct idr *idr, int id)
{
	idr_remove(idr, id);
}
