/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file port_api.c BFA FCS port
 */

#include <fcs/bfa_fcs.h>
#include <fcs/bfa_fcs_port.h>
#include <fcs/bfa_fcs_rport.h>
#include "fcs_rport.h"
#include "fcs_fabric.h"
#include "fcs_trcmod.h"

BFA_TRC_FILE(FCS, PORT_API);

/**
 * @}
 */

/**
 * @dg fcs_port_api BFA FCS port API
 * @{
 */

void
bfa_fcs_cfg_base_port(bfa_fcs_t *fcs, bfa_port_cfg_t *port_cfg)
{
}

bfa_fcs_port_t *
bfa_fcs_get_base_port(bfa_fcs_t *fcs)
{
	return (&fcs->fabric.bport);
}

wwn_t
bfa_fcs_port_get_rport(bfa_fcs_port_t *port,
							wwn_t wwn, 
							int index,
							int nrports, 
							bfa_boolean_t bwwn)
{
	bfa_q_t        *qh, *qe;
	bfa_fcs_rport_t *rport = NULL;
	int             i;
	bfa_fcs_t      *fcs;

	if (port == NULL || nrports == 0)
		return (wwn_t)0;

	fcs = port->fcs;
	bfa_trc(fcs, (uint32_t) nrports);

	i = 0;
	qh = &port->rport_q;
	qe = bfa_q_first(qh);

	while ((qe != qh) && (i < nrports)) {
		rport = (bfa_fcs_rport_t *) qe;
		if (bfa_os_ntoh3b(rport->pid) > 0xFFF000) {
			qe = bfa_q_next(qe);
			bfa_trc(fcs, (uint32_t) rport->pwwn);
			bfa_trc(fcs, rport->pid);
			bfa_trc(fcs, i);
			continue;
		}

		if(bwwn) {
			if (!memcmp(&wwn, &rport->pwwn, 8)) 
				break;
		}
		else {
			if (i == index)
				break;
		}

		i++;
		qe = bfa_q_next(qe);
	}

	bfa_trc(fcs, i);
	return rport->pwwn;
}

void
bfa_fcs_port_get_rports(bfa_fcs_port_t *port, wwn_t rport_wwns[], int *nrports)
{
	bfa_q_t        *qh, *qe;
	bfa_fcs_rport_t *rport = NULL;
	int             i;
	bfa_fcs_t      *fcs;

	if (port == NULL || rport_wwns == NULL || *nrports == 0)
		return;

	fcs = port->fcs;
	bfa_trc(fcs, (uint32_t) *nrports);

	i = 0;
	qh = &port->rport_q;
	qe = bfa_q_first(qh);

	while ((qe != qh) && (i < *nrports)) {
		rport = (bfa_fcs_rport_t *) qe;
		if (bfa_os_ntoh3b(rport->pid) > 0xFFF000) {
			qe = bfa_q_next(qe);
			bfa_trc(fcs, (uint32_t) rport->pwwn);
			bfa_trc(fcs, rport->pid);
			bfa_trc(fcs, i);
			continue;
		}

		rport_wwns[i] = rport->pwwn;

		i++;
		qe = bfa_q_next(qe);
	}

	bfa_trc(fcs, i);
	*nrports = i;
	return;
}

/*
 * Iterate's through all the rport's in the given port to
 * determine the maximum operating speed.
 */
bfa_pport_speed_t
bfa_fcs_port_get_rport_max_speed(bfa_fcs_port_t *port)
{
	bfa_q_t        *qh, *qe;
	bfa_fcs_rport_t *rport = NULL;
	bfa_fcs_t      *fcs;
	bfa_pport_speed_t max_speed =0;
	bfa_pport_attr_t  pport_attr;
	bfa_pport_speed_t pport_speed;

	if (port == NULL)
		return 0;

	fcs = port->fcs;

	/* Get Physical port's current speed */
	bfa_pport_get_attr(port->fcs->bfa, &pport_attr);
	pport_speed = pport_attr.speed;
	bfa_trc(fcs, pport_speed);

	qh = &port->rport_q;
	qe = bfa_q_first(qh);

	while (qe != qh) {
		rport = (bfa_fcs_rport_t *) qe;
		if ((bfa_os_ntoh3b(rport->pid) > 0xFFF000) ||
			(bfa_fcs_rport_get_state(rport) ==  BFA_RPORT_OFFLINE)) {
			qe = bfa_q_next(qe);
			continue;
		}

		if	((rport->rpf.rpsc_speed  == BFA_PPORT_SPEED_8GBPS) ||
			(rport->rpf.rpsc_speed > pport_speed)) {
			max_speed = rport->rpf.rpsc_speed;
			break;
		}
		else if (rport->rpf.rpsc_speed > max_speed) {
			max_speed = rport->rpf.rpsc_speed;
		}

		qe = bfa_q_next(qe);
	}

	bfa_trc(fcs, max_speed);
	return max_speed;
}

bfa_fcs_port_t *
bfa_fcs_lookup_port(bfa_fcs_t *fcs, uint16_t vf_id, wwn_t lpwwn)
{
	bfa_fcs_vport_t *vport;
	bfa_fcs_vf_t   *vf;

	bfa_assert(fcs != NULL);

	vf = bfa_fcs_vf_lookup(fcs, vf_id);
	if (vf == NULL) {
		bfa_trc(fcs, vf_id);
		return (NULL);
	}

	if (!lpwwn || (vf->bport.port_cfg.pwwn == lpwwn))
		return (&vf->bport);

	vport = bfa_fcs_fabric_vport_lookup(vf, lpwwn);
	if (vport)
		return (&vport->lport);

	return (NULL);
}

/*
 *  API corresponding to VmWare's NPIV_VPORT_GETINFO.
 */
void
bfa_fcs_port_get_info(bfa_fcs_port_t *port, bfa_port_info_t *port_info)
{

	bfa_trc(port->fcs, port->fabric->fabric_name);

	if (port->vport == NULL) {
		/*
		 * This is a Physical port
		 */
		port_info->port_type = BFA_PORT_TYPE_PHYSICAL;

		/*
		 * @todo : need to fix the state & reason
		 */
		port_info->port_state = 0;
		port_info->offline_reason = 0;

		port_info->port_wwn = bfa_fcs_port_get_pwwn(port);
		port_info->node_wwn = bfa_fcs_port_get_nwwn(port);

		port_info->max_vports_supp = BFA_FCS_MAX_VPORTS_SUPP;
		port_info->num_vports_inuse =
			bfa_fcs_fabric_vport_count(port->fabric);
		port_info->max_rports_supp = BFA_FCS_MAX_RPORTS_SUPP;
		port_info->num_rports_inuse = port->num_rports;
	} else {
		/*
		 * This is a virtual port
		 */
		port_info->port_type = BFA_PORT_TYPE_VIRTUAL;

		/*
		 * @todo : need to fix the state & reason
		 */
		port_info->port_state = 0;
		port_info->offline_reason = 0;

		port_info->port_wwn = bfa_fcs_port_get_pwwn(port);
		port_info->node_wwn = bfa_fcs_port_get_nwwn(port);
	}
}

void
bfa_fcs_port_get_stats(bfa_fcs_port_t *fcs_port, bfa_port_stats_t *port_stats)
{
	bfa_os_memcpy(port_stats, &fcs_port->stats, sizeof(bfa_port_stats_t));
	return;
}

void
bfa_fcs_port_clear_stats(bfa_fcs_port_t *fcs_port)
{
	bfa_os_memset(&fcs_port->stats, 0, sizeof(bfa_port_stats_t));
	return;
}

void
bfa_fcs_port_enable_ipfc_roles(bfa_fcs_port_t *fcs_port)
{
	fcs_port->port_cfg.roles |= BFA_PORT_ROLE_FCP_IPFC;
	return;
}

void
bfa_fcs_port_disable_ipfc_roles(bfa_fcs_port_t *fcs_port)
{
	fcs_port->port_cfg.roles &= ~BFA_PORT_ROLE_FCP_IPFC;
	return;
}

/**
 * @}
 */
