/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_DIAG_H__
#define __HAL_DIAG_H__

#include <bfa_diag.h>
#include <bfi/bfi_diag.h>

/**
 * DIAG test parameter and result data structure
 */

typedef struct {
	bfa_diag_fwping_result_t *result;
	bfa_cb_diag_t  cbfn;
	void           *cbarg;
	bfa_timer_t timer;
	uint32_t        data;
	uint8_t         vcidx;	/* keep track of vc */
	uint8_t         lock;
	uint8_t         timer_active;
	uint8_t         rsv[1];
	bfa_cb_qe_t    hcb_qe;
	uint32_t        status;
	uint32_t        count;
} bfa_diag_fwping_t;

/*
 * temp sensor has it's own cbfn cause it can co-exist with other diag cmd
 */
typedef struct {
	bfa_cb_diag_t  cbfn;
	void           *cbarg;
	uint16_t       *temp;
	uint8_t         lock;
	uint8_t         rsv[3];
	bfa_cb_qe_t    hcb_qe;
	uint32_t        status;
} bfa_diag_tsensor_t;

typedef struct {
	sfp_mem_t      *sfpmem;
	bfa_cb_diag_t  cbfn;
	void           *cbarg;
	uint8_t         lock;
	uint8_t         static_data;
	uint8_t         rsv[2];
	bfa_cb_qe_t    hcb_qe;
	uint32_t        status;
} bfa_diag_sfpshow_t;

typedef struct {
	bfa_cb_diag_t  cbfn;
	void           *cbarg;
	bfa_cb_qe_t    hcb_qe;
	void           *result;
	uint8_t         lock;
	uint8_t         rsv[3];
	uint32_t        status;
} bfa_diag_lb_t;
	
typedef struct bfa_diag_mod {
	bfa_t      *bfa;		/*! Back pointer to HAL */
	bfa_diag_fwping_t fwping;
	bfa_diag_tsensor_t tsensor;
	bfa_diag_sfpshow_t sfpshow;
	bfa_diag_lb_t lb;
	void           *result;		/* store result pointer */
	bfa_timer_t timer;
	uint8_t        *dbuf_kva;	/*!< dma buf virtual address */
	uint64_t        dbuf_pa;	/*!< dma buf physical address */
	bfa_cb_diag_t  cbfn;
	void           *cbarg;
	uint8_t         block;
	uint8_t         timer_active;
	uint8_t         rsvd[2];

	bfa_boolean_t   beacon;		/*!< port beacon status */
	bfa_boolean_t   link_e2e_beacon;/*!< link beacon status */
	bfa_cb_qe_t    hcb_qe;
	uint32_t        status;
} bfa_diag_mod_t;
#define BFA_DIAG_MOD(__bfa)     (&(__bfa)->modules.diag_mod)

void            diag_intr(bfa_t *bfa, bfi_msg_t *msg);

#endif /* __HAL_DIAG_H__ */
