/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#include <bfa.h>
#include <log/bfa_log_hal.h>

BFA_TRC_FILE(HAL, FCPIM);
BFA_MODULE(fcpim);

/**
 * @dg hal_fcpim_mod HAL FCP Initiator Mode module
 * @{
 */

/**
 * @brief
 * 		Compute and return memory needed by FCP(im) module.
 */
static void
bfa_fcpim_meminfo(bfa_ioc_cfg_t *cfg, uint32_t *km_len, uint32_t *dm_len)
{
	bfa_itnim_meminfo(cfg, km_len, dm_len);

	/**
	 * IO memory
	 */
	if (cfg->fwcfg.num_ioim_reqs < BFA_IOIM_MIN)
		cfg->fwcfg.num_ioim_reqs = BFA_IOIM_MIN;
	*km_len += cfg->fwcfg.num_ioim_reqs *
	  (sizeof(bfa_ioim_t) + sizeof(struct bfa_ioim_sp_s));

	*dm_len += cfg->fwcfg.num_ioim_reqs * BFI_IOIM_SNSLEN;

	/**
	 * task management command memory
	 */
	if (cfg->fwcfg.num_tskim_reqs < BFA_TSKIM_MIN)
		cfg->fwcfg.num_tskim_reqs = BFA_TSKIM_MIN;
	*km_len += cfg->fwcfg.num_tskim_reqs * sizeof(bfa_tskim_t);
}

/**
 * @brief
 *
 */
static void
bfa_fcpim_attach(bfa_t *bfa, void *bfad, bfa_ioc_cfg_t *cfg,
		     bfa_meminfo_t *meminfo, bfa_pcidev_t *pcidev)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	bfa_trc(bfa, cfg->mpio_mode);
	bfa_trc(bfa, cfg->path_tov);
	bfa_trc(bfa, cfg->fwcfg.num_itnims);
	bfa_trc(bfa, cfg->fwcfg.num_ioim_reqs);
	bfa_trc(bfa, cfg->fwcfg.num_tskim_reqs);

	fcpim->bfa            = bfa;
	fcpim->num_itnims     = cfg->fwcfg.num_itnims;
	fcpim->num_ioim_reqs  = cfg->fwcfg.num_ioim_reqs;
	fcpim->num_tskim_reqs = cfg->fwcfg.num_tskim_reqs;
	fcpim->mpio_mode      = cfg->mpio_mode;
	fcpim->path_tov       = cfg->path_tov;
	fcpim->delay_comp	  = cfg->delay_comp;

	bfa_itnim_attach(fcpim, meminfo);
	bfa_tskim_attach(fcpim, meminfo);
	bfa_ioim_attach(fcpim, meminfo);
}

static void
bfa_fcpim_initdone(bfa_t *bfa)
{
}

static void
bfa_fcpim_detach(bfa_t *bfa)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	bfa_ioim_detach(fcpim);
	bfa_tskim_detach(fcpim);
}

static void
bfa_fcpim_start(bfa_t *bfa)
{
}

static void
bfa_fcpim_stop(bfa_t *bfa)
{
}

static void
bfa_fcpim_iocdisable(bfa_t *bfa)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);
	bfa_itnim_t *itnim;
	bfa_q_t        *qe, *qen;

	bfa_q_iter_safe(&fcpim->itnim_q, qe, qen) {
		itnim = (bfa_itnim_t *) qe;
		bfa_itnim_iocdisable(itnim);
	}
}

void
bfa_fcpim_mpio_mode_set(bfa_t *bfa, bfa_boolean_t mpio_mode,
			    uint16_t path_tov)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	fcpim->mpio_mode = mpio_mode;
	if (!mpio_mode) {
		if (path_tov)
			fcpim->path_tov = path_tov * 1000;
		else
			fcpim->path_tov = BFA_FCPIM_PATH_TOV_DEF;
	}
}

bfa_boolean_t
bfa_fcpim_mpio_mode_get(bfa_t *bfa)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	return (fcpim->mpio_mode);
}

uint16_t
bfa_fcpim_path_tov_get(bfa_t *bfa)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	return (fcpim->path_tov / 1000);
}

void
bfa_fcpim_qdepth_set(bfa_t *bfa, uint16_t q_depth)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	bfa_assert(q_depth <= BFA_IOC_MAX_Q_DEPTH);

	fcpim->q_depth = q_depth;
}

uint16_t
bfa_fcpim_qdepth_get(bfa_t *bfa)
{
	struct bfa_fcpim_mod_s *fcpim = BFA_FCPIM_MOD(bfa);

	return (fcpim->q_depth);
}

/**
 * @}
 */
