/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_FCPIM_H__
#define __HAL_FCPIM_H__

#include <bfa_fcpim.h>
#include <cs/bfa_wc.h>
#include "bfa_sgpg_priv.h"

#define BFA_ITNIM_MIN   4
#define BFA_ITNIM_MAX   1024

#define BFA_IOIM_MIN    8
#define BFA_IOIM_MAX    2000

#define BFA_TSKIM_MIN   8
#define BFA_TSKIM_MAX   512
#define BFA_FCPIM_PATH_TOV_DEF	(30 * 1000)	/* in millisecs */

struct bfa_fcpim_stats_s {
	uint32_t        total_ios;	/*!< Total IO count */
	uint32_t        qresumes;	/*!< IO waiting for CQ space */
	uint32_t        no_iotags;	/*!< NO IO contexts */
	uint32_t        io_aborts;	/*!< IO abort requests */
	uint32_t        no_tskims;	/*!< NO task management contexts */
	uint32_t        iocomp_ok;	/*!< IO completions with OK status */
	uint32_t        iocomp_underrun;	/*!< IO underrun (good) */
	uint32_t        iocomp_overrun;	/*!< IO overrun (good) */
	uint32_t        iocomp_aborted;	/*!< Aborted IO requests */
	uint32_t        iocomp_timedout;	/*!< IO timeouts */
	uint32_t        iocom_nexus_abort;	/*!< IO selection timeouts */
	uint32_t        iocom_proto_err;	/*!< IO protocol errors */
	uint32_t        iocom_dif_err;	/*!< IO SBC-3 protection errors */
	uint32_t        iocom_tm_abort;	/*!< IO aborted by TM requests */
	uint32_t        iocom_sqer_needed;	/*!< IO retry for SQ error
						 *recovery */
	uint32_t        iocom_res_free;	/*!< Delayed freeing of IO resources */
	uint32_t        iocomp_scsierr;	/*!< IO with non-good SCSI status */
	uint32_t        iocom_hostabrts;	/*!< Host IO abort requests */
	uint32_t        iocom_utags;	/*!< IO comp with unknown tags */
	uint32_t        io_cleanups;	/*!< IO implicitly aborted */
	uint32_t        io_tmaborts;	/*!< IO aborted due to TM commands */
	uint32_t        rsvd;
};
#define bfa_fcpim_stats(__fcpim, __stats)   \
    (__fcpim)->stats.__stats ++

struct bfa_fcpim_mod_s {
	bfa_t		*bfa;
	bfa_itnim_t		*itnim_arr;
	bfa_ioim_t		*ioim_arr;
	struct bfa_ioim_sp_s *ioim_sp_arr;
	bfa_tskim_t		*tskim_arr;
	struct bfa_dma_s	snsbase;
	int			num_itnims;
	int			num_ioim_reqs;
	int			num_tskim_reqs;
	bfa_boolean_t		mpio_mode;
	uint32_t		path_tov;
	uint16_t		q_depth;
	uint16_t		rsvd;
	bfa_q_t			itnim_q;        /*!< queue of active itnim    */
	bfa_q_t			ioim_free_q;    /*!< free IO resources        */
	bfa_q_t			ioim_resfree_q; /*!< IOs waiting for f/w      */
	bfa_q_t			ioim_comp_q;    /*!< IO global comp Q         */
	bfa_q_t			tskim_free_q;
	uint32_t		ios_active;	/*!< current active IOs	      */
	uint32_t		delay_comp;
	struct bfa_fcpim_stats_s stats;
};

struct bfa_ioim_s;
struct bfa_tskim_s;

/**
 * HAL IO (initiator mode)
 */
struct bfa_ioim_s {
	bfa_q_t			qe;		/*!< queue elememt            */
	bfa_sm_t		sm;		/*!< HAL ioim state machine   */
	bfa_t		*bfa;		/*!< HAL module               */
	struct bfa_fcpim_mod_s	*fcpim;		/*!< parent fcpim module      */
	bfa_itnim_t		*itnim;		/*!< i-t-n nexus for this IO  */
	bfad_ioim_t		*dio;		/*!< driver IO handle         */
	uint16_t		iotag;		/*!< FWI IO tag               */
	uint16_t		abort_tag;	/*!< unqiue abort request tag */
	uint16_t		nsges;		/*!< number of SG elements    */
	uint16_t		nsgpgs;		/*!< number of SG pages       */
	bfa_sgpg_t		*sgpg;		/*!< first SG page            */
	bfa_q_t			sgpg_q;		/*!< allocated SG pages       */
	bfa_cb_qe_t		hcb_qe;		/*!< bfa callback qelem       */
	bfa_cb_cbfn_t		io_cbfn;	/*!< IO completion handler    */
	struct bfa_ioim_sp_s *iosp;		/*!< slow-path IO handling    */
};

struct bfa_ioim_sp_s {
	bfi_msg_t		comp_rspmsg;	/*!< IO comp f/w response     */
	uint8_t			*snsinfo;	/*!< sense info for this IO   */
	bfa_sgpg_wqe_t	sgpg_wqe;	/*!< waitq elem for sgpg      */
	struct bfa_reqq_wait_s reqq_wait;	/*!< to wait for room in reqq */
	bfa_boolean_t		abort_explicit;	/*!< aborted by OS            */
	struct bfa_tskim_s	*tskim;		/*!< Relevant TM cmd          */
};

/**
 * HAL Task management command (initiator mode)
 */
struct bfa_tskim_s {
	bfa_q_t              qe;
	bfa_sm_t             sm;
	bfa_t            *bfa;           /*!< HAL module                  */
	struct bfa_fcpim_mod_s  *fcpim;  /*!< parent fcpim module         */
	bfa_itnim_t      *itnim;         /*!< i-t-n nexus for this IO     */
	bfad_tskim_t         *dtsk;          /*!< driver task mgmt cmnd       */
	bfa_boolean_t        notify;         /*!< notify itnim on TM comp     */
	lun_t                lun;            /*!< lun if applicable           */
	fcp_tm_cmnd_t        tm_cmnd;        /*!< task management command     */
	uint16_t             tsk_tag;        /*!< FWI IO tag                  */
	uint8_t              tsecs;          /*!< timeout in seconds          */
	struct bfa_reqq_wait_s  reqq_wait;   /*!< to wait for room in reqq    */
	bfa_q_t              io_q;           /*!< queue of affected IOs       */
	bfa_wc_t             wc;             /*!< waiting counter             */
	bfa_cb_qe_t         hcb_qe;         /*!< bfa callback qelem          */
	bfi_tskim_status_t   tsk_status;     /*!< TM status                   */
};

/**
 * HAL i-t-n (initiator mode)
 */
struct bfa_itnim_s {
	bfa_q_t         qe;		/*!< queue element               */
	bfa_sm_t        sm;		/*!< i-t-n im HAL state machine  */
	bfa_t      *bfa;		/*!< bfa instance                */
	bfa_rport_t *rport;		/*!< bfa rport                   */
	void           *ditn;		/*!< driver i-t-n structure      */
	bfi_mhdr_t      mhdr;		/*!< pre-built mhdr              */
	uint8_t         msg_no;		/*!< itnim/rport firmware handle */
	uint8_t         reqq;		/*!< CQ for requests             */
	bfa_cb_qe_t    hcb_qe;		/*!< bfa callback qelem          */
	bfa_q_t         pending_q;	/*!< queue of pending IO requests*/
	bfa_q_t         io_q;		/*!< queue of active IO requests */
	bfa_q_t         io_cleanup_q;	/*!< IO being cleaned up         */
	bfa_q_t         tsk_q;		/*!< queue of active TM commands */
	bfa_q_t         delay_comp_q;/*!< queue of failed inflight cmds */
	bfa_boolean_t   seq_rec;	/*!< SQER supported              */
	bfa_boolean_t   is_online;	/*!< itnim is ONLINE for IO      */
	bfa_boolean_t   iotov_active;	/*!< IO TOV timer is active	 */
	bfa_wc_t        wc;		/*!< waiting counter             */
	bfa_timer_t timer;		/*!< pending IO TOV		 */
	struct bfa_reqq_wait_s reqq_wait; /*!< to wait for room in reqq */
	struct bfa_fcpim_mod_s *fcpim;	/*!< fcpim module                */
	bfa_itnim_hal_stats_t	stats;
};

#define bfa_itnim_is_online(_itnim) (_itnim)->is_online
#define BFA_FCPIM_MOD(_hal) (&(_hal)->modules.fcpim_mod)
#define BFA_IOIM_FROM_TAG(_fcpim, _iotag)               \
    (&fcpim->ioim_arr[_iotag & (fcpim->num_ioim_reqs - 1)])
#define BFA_TSKIM_FROM_TAG(_fcpim, _tmtag)                  \
    (&fcpim->tskim_arr[_tmtag & (fcpim->num_tskim_reqs - 1)])

/*
 * function prototypes
 */
void            bfa_ioim_attach(struct bfa_fcpim_mod_s *fcpim,
				    bfa_meminfo_t *minfo);
void            bfa_ioim_detach(struct bfa_fcpim_mod_s *fcpim);
void            bfa_ioim_isr(bfa_t *bfa, bfi_msg_t *msg);
void            bfa_ioim_good_comp_isr(bfa_t *bfa, bfi_msg_t *msg);
void            bfa_ioim_cleanup(bfa_ioim_t *ioim);
void            bfa_ioim_cleanup_tm(bfa_ioim_t *ioim,
					bfa_tskim_t *tskim);
void            bfa_ioim_iocdisable(bfa_ioim_t *ioim);
void            bfa_ioim_tov(bfa_ioim_t *ioim);

void            bfa_tskim_attach(struct bfa_fcpim_mod_s *fcpim,
				     bfa_meminfo_t *minfo);
void            bfa_tskim_detach(struct bfa_fcpim_mod_s *fcpim);
void            bfa_tskim_isr(bfa_t *bfa, bfi_msg_t *msg);
void            bfa_tskim_iodone(bfa_tskim_t *tskim);
void            bfa_tskim_iocdisable(bfa_tskim_t *tskim);
void            bfa_tskim_cleanup(bfa_tskim_t *tskim);

void            bfa_itnim_meminfo(bfa_ioc_cfg_t *cfg, uint32_t *km_len,
				      uint32_t *dm_len);
void            bfa_itnim_attach(struct bfa_fcpim_mod_s *fcpim,
				     bfa_meminfo_t *minfo);
void            bfa_itnim_detach(struct bfa_fcpim_mod_s *fcpim);
void            bfa_itnim_iocdisable(bfa_itnim_t *itnim);
void            bfa_itnim_isr(bfa_t *bfa, bfi_msg_t *msg);
void            bfa_itnim_iodone(bfa_itnim_t *itnim);
void            bfa_itnim_tskdone(bfa_itnim_t *itnim);
void		bfa_itnim_qresume(void *cbarg);
bfa_boolean_t   bfa_itnim_hold_io(bfa_itnim_t *itnim);

#endif /* __HAL_FCPIM_H__ */

