/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_INTR_H__
#define __HAL_INTR_H__

/**
 * Message handler
 */
typedef void (*bfa_isr_func_t) (bfa_t *bfa, bfi_msg_t *m);
void bfa_isr_unhandled(bfa_t *bfa, bfi_msg_t *m);
void bfa_isr_bind(bfi_mclass_t mc, bfa_isr_func_t isr_func);


#define bfa_reqq_pi(__bfa, __reqq)	(__bfa)->ioc.req_cq_pi[__reqq]
#define bfa_reqq_ci(__bfa, __reqq)					\
	*(volatile uint32_t *)((__bfa)->ioc.req_cq_shadow_ci[__reqq].kva)

#define bfa_reqq_full(__bfa, __reqq)				\
	(((bfa_reqq_pi(__bfa, __reqq) + 1) &			\
	  ((__bfa)->ioc.cfg.num_reqq_elems - 1)) ==			\
	 bfa_reqq_ci(__bfa, __reqq))

#define bfa_reqq_next(__bfa, __reqq)				\
	((void *)((bfi_msg_t *)((__bfa)->ioc.req_cq_ba[__reqq].kva)	\
			  + bfa_reqq_pi((__bfa), (__reqq))))

#define bfa_reqq_ionext(__bfa, __reqq)				\
	(bfa_reqq_full(__bfa, __reqq) ? NULL :			\
	 ((void *)((bfi_msg_t *)((__bfa)->ioc.req_cq_ba[__reqq].kva)	\
			  + bfa_reqq_pi((__bfa), (__reqq)))))

#define bfa_reqq_produce(__bfa, __reqq)	do {			\
	(__bfa)->ioc.req_cq_pi[__reqq]++;				\
	(__bfa)->ioc.req_cq_pi[__reqq] &=				\
				((__bfa)->ioc.cfg.num_reqq_elems - 1);	\
	bfa_reg_write((__bfa)->ioc.bfa_regs.cpe_q_pi[__reqq],	\
					(__bfa)->ioc.req_cq_pi[__reqq]);\
	bfa_os_mmiowb();						\
} while (0)

#define bfa_rspq_pi(__bfa, __rspq)					\
	*(volatile uint32_t *)((__bfa)->ioc.rsp_cq_shadow_pi[__rspq].kva)

#define bfa_rspq_ci(__bfa, __rspq)	(__bfa)->ioc.rsp_cq_ci[__rspq]
#define bfa_rspq_elem(__bfa, __rspq, __ci)				\
	&((bfi_msg_t *)((__bfa)->ioc.rsp_cq_ba[__rspq].kva))[__ci]

#define CQ_INCR(__index, __size)					\
			(__index)++; (__index) &= ((__size) - 1)

/**
 * @brief
 * Queue element to wait for room in request queue. FIFO order is
 * maintained when fullfilling requests.
 */
struct bfa_reqq_wait_s {
	bfa_q_t		qe;
	void		(*qresume) (void *cbarg);
	void		*cbarg;
};

/**
 * Circular queue usage assignments
 */
enum {
	BFA_REQQ_IOC	= 0,	/* !< all low-priority IOC msgs	*/
	BFA_REQQ_FCXP	= 0,	/* !< all FCXP messages		*/
	BFA_REQQ_PORT	= 0,	/* !< all port messages		*/
	BFA_REQQ_FLASH	= 0,	/* !< for flash module		*/
	BFA_REQQ_DIAG	= 0,	/* !< for diag module		*/
	BFA_REQQ_RPORT	= 0,	/* !< all port messages		*/
	BFA_REQQ_SBOOT	= 0,	/* !< all san boot messages	*/
	BFA_REQQ_QOS_LO	= 1,	/* !< all low priority IO	*/
	BFA_REQQ_QOS_MD	= 2,	/* !< all medium priority IO	*/
	BFA_REQQ_QOS_HI	= 3,	/* !< all high priority IO	*/
};

static inline void
bfa_reqq_winit(struct bfa_reqq_wait_s *wqe, void (*qresume) (void *cbarg),
			void *cbarg)
{
	wqe->qresume = qresume;
	wqe->cbarg = cbarg;
}

#define bfa_reqq(__bfa, __reqq)	&(__bfa)->reqq_waitq[__reqq]

/**
 * static inline void
 * bfa_reqq_wait(bfa_t *bfa, int reqq, struct bfa_reqq_wait_s *wqe)
 */
#define bfa_reqq_wait(__bfa, __reqq, __wqe) do {			\
									\
		bfa_q_t	*waitq = bfa_reqq(__bfa, __reqq);		\
									\
		bfa_assert(((__reqq) < BFI_IOC_MAX_CQS));		\
		bfa_assert((__wqe)->qresume && (__wqe)->cbarg);		\
									\
		bfa_q_enq(waitq, (__wqe));				\
} while (0)

#define bfa_reqq_wcancel(__wqe)	bfa_q_qe_deq(__wqe)

#endif /* __HAL_INTR_H__ */
