/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_IOC_H__
#define __BFA_IOC_H__

#include <cs/bfa_sm.h>
#include <bfi/bfi.h>
#include <bfi/bfi_ioc.h>
#include <bfi/bfi_iocfc.h>
#include <defs/bfa_defs_types.h>
#include <bfi/bfi_boot.h>
#include "bfa_reg_priv.h"
#include "bfa_callback_priv.h"

#define BFA_REQQ_NELEMS_MIN		(4)
#define BFA_RSPQ_NELEMS_MIN		(4)

/**
 * Structure used to remember the DMA-able memory block's KVA and Physical
 * Address
 */
struct bfa_dma_s {
	void		*kva;	/*! Kernel virtual address	*/
	uint64_t	pa;	/*! Physical address		*/
};

#define BFA_DMA_ALIGN_SZ	256
#define BFA_ROUNDUP(_l, _s)	(((_l) + ((_s) - 1)) & ~((_s) - 1))

struct bfa_ioc_s {
	bfa_sm_t        	sm;
	bfa_t		*bfa;	/*! Back pointer to HAL */
	union {
		bfi_iocfc_cfg_reply_t *cfg_reply;
	} event_args;

	bfa_pcidev_t	pci_info;
	bfa_ioc_cfg_t		cfg;

	uint32_t        	req_cq_pi[BFI_IOC_MAX_CQS];
	uint32_t        	rsp_cq_ci[BFI_IOC_MAX_CQS];
	bfi_ioc_i2h_msg_t 	mbox_msg;
					/*!< Used to store msg from the MBOX */

	bfa_cb_qe_t		init_hcb_qe;
	bfa_cb_qe_t		stop_hcb_qe;
	bfa_cb_qe_t		dis_hcb_qe;
	bfa_cb_qe_t		stats_hcb_qe;
	bfa_boolean_t		cfgdone;
	bfa_boolean_t		init_cb;
	bfa_boolean_t		stop_cb;
	bfa_boolean_t		disable_cb;
	bfa_boolean_t		ioc_started;
	bfa_timer_t		ioc_timer;
	bfa_timer_t		sem_timer;
	uint32_t		hb_count;
	uint32_t		hb_fail;
	uint32_t		hwinit_count;

	void			*dbg_fwsave;
	int			dbg_fwsave_len;

	struct bfa_dma_s		cfg_info;
	bfi_iocfc_cfg_t	*cfginfo;
	struct bfa_dma_s		cfgrsp_dma;
	bfi_iocfc_cfgrsp_t	*cfgrsp;

	uint8_t			*stats_kva;
	uint64_t		stats_pa;
	bfa_fw_stats_t		*fw_stats;
	bfa_timer_t		stats_timer;	/*!< timer */
	bfa_ioc_stats_t		*stats_ret;	/*!< driver stats location */
	bfa_status_t		stats_status;	/*!< stats/statsclr status */
	bfa_boolean_t   	stats_busy;	/*!< outstanding stats */
	bfa_cb_ioc_t   	stats_cbfn;	/*!< driver callback function */
	void           		*stats_cbarg;	/*!< user callback arg */
	bfa_ioc_drv_stats_t	stats;

	struct bfa_dma_s   	req_cq_ba[BFI_IOC_MAX_CQS];
	struct bfa_dma_s   	req_cq_shadow_ci[BFI_IOC_MAX_CQS];
	struct bfa_dma_s   	rsp_cq_ba[BFI_IOC_MAX_CQS];
	struct bfa_dma_s   	rsp_cq_shadow_pi[BFI_IOC_MAX_CQS];
	bfa_chip_regs_t 	bfa_regs;	/*!< HAL device registers */
};

#define bfa_ioc_lpuid(__bfa)	(__bfa)->ioc.pci_info.pci_func
#define bfa_ioc_maxfrsize(__bfa)	(__bfa)->ioc.cfgrsp->maxfrsize
#define bfa_ioc_rx_bbcredit(__bfa)	(__bfa)->ioc.cfgrsp->rx_bbcredit
#define bfa_ioc_speed_sup(__bfa)	\
	BFI_ADAPTER_GETP(SPEED, (__bfa)->ioc.cfgrsp->adapter_prop)

void bfa_ioc_attach(struct bfa_ioc_s *ioc, bfa_pcidev_t *pcidev);
void bfa_ioc_init(struct bfa_ioc_s *ioc);
void bfa_ioc_start(struct bfa_ioc_s *ioc);
void bfa_ioc_stop(struct bfa_ioc_s *ioc);
void bfa_ioc_enable(struct bfa_ioc_s *ioc);
void bfa_ioc_disable(struct bfa_ioc_s *ioc);

void bfa_ioc_reg_init(struct bfa_ioc_s *ioc, bfa_pcidev_t *pcidev);
void bfa_ioc_boot(bfa_t *bfa, uint32_t boot_type, uint32_t boot_param);
void bfa_ioc_isr(struct bfa_ioc_s *ioc, bfi_ioc_i2h_msg_t *msg);
void bfa_ioc_msgget(struct bfa_ioc_s *ioc, bfi_ioc_i2h_msg_t *msg);
bfa_status_t bfa_ioc_pll_init(bfa_t *bfa);
bfa_boolean_t bfa_ioc_is_operational(bfa_t *bfa);
bfa_boolean_t bfa_ioc_adapter_is_disabled(bfa_t *bfa);
void bfa_ioc_mbox_send(struct bfa_ioc_s *ioc, void *ioc_msg, int len);
void bfa_ioc_cfg_complete(struct bfa_ioc_s *ioc);

#endif /* __BFA_IOC_H__ */

