/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_PORT_H__
#define __HAL_PORT_H__

#include <defs/bfa_defs_pport.h>
#include <bfi/bfi_pport.h>
#include "bfa_intr_priv.h"

/**
 * HAL physical port data structure
 */
typedef struct {
	bfa_t		*bfa;	/*!< parent HAL instance */
	bfa_sm_t		sm;	/*!< port state machine */
	wwn_t			nwwn;	/*!< node wwn of physical port */
	wwn_t			pwwn;	/*!< port wwn of physical oprt */
	bfa_pport_speed_t	speed_sup;
					/*!< supported speeds */
	bfa_pport_speed_t	speed;	/*!< current speed */
	bfa_pport_topology_t	topology;	/*!< current topology */
	uint8_t			myalpa;	/*!< my ALPA in LOOP topology */
	uint8_t			rsvd[3];
	bfa_pport_cfg_t		cfg;	/*!< current port configuration */
	bfa_qos_attr_t  qos_attr;   /* QoS Attributes */
	bfa_qos_vc_attr_t qos_vc_attr;  /*!< VC info from ELP */
	struct bfa_reqq_wait_s	reqq_wait;
					/*!< to wait for room in reqq */
	struct bfa_reqq_wait_s	svcreq_wait;
					/*!< to wait for room in reqq */
	struct bfa_reqq_wait_s	stats_reqq_wait;
					/*!< to wait for room in reqq (stats) */
	void			*event_cbarg;
	void			(*event_cbfn) (void *cbarg,
						bfa_pport_event_t event);
	union {
		bfi_pport_i2h_msg_t i2hmsg;
	} event_arg;
	void			*bfad;	/*!< BFA driver handle */
	bfa_cb_qe_t		hcb_qe;	/*!< HAL callback queue elem */
	bfa_pport_linkstate_t	hcb_event;
					/*!< link event for callback */
	uint32_t		msgtag;	/*!< fimrware msg tag for reply */
	uint8_t			*stats_kva;
	uint64_t		stats_pa;
	bfa_pport_stats_t	*stats;	/*!< pport stats */
	bfa_timer_t		timer;	/*!< timer */
	bfa_pport_stats_t 	*stats_ret;
					/*!< driver stats location */
	bfa_status_t		stats_status;
					/*!< stats/statsclr status */
	bfa_boolean_t   	stats_busy;
					/*!< outstanding stats/statsclr */
	bfa_boolean_t   	diag_busy;
					/*!< diag busy status */
	bfa_boolean_t   	beacon;
					/*!< port beacon status */
	bfa_boolean_t   	link_e2e_beacon;
					/*!< link beacon status */
	bfa_cb_pport_t 	stats_cbfn;
					/*!< driver callback function */
	void			*stats_cbarg;
					/* *!< user callback arg */
} bfa_pport_t;

#define BFA_PORT_MOD(__bfa)	(&(__bfa)->modules.pport)

/*
 * public functions
 */
void	bfa_pport_isr(bfa_t *bfa, bfi_msg_t *msg);
#endif /* __HAL_PORT_H__ */
