/*
 * Copyright (c) 2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __HAL_PRIV_H__
#define __HAL_PRIV_H__

#include "bfa_iocfc.h"
#include "bfa_timer_priv.h"
#include "bfa_intr_priv.h"
#include "bfa_reg_priv.h"
#include "bfa_trcmod_priv.h"
#include "bfa_modules_priv.h"
#include "bfa_fwimg_priv.h"
#include <cs/bfa_log.h>

/**
 * @brief
 * Macro to define a new HAL module
 */
#define BFA_MODULE(__mod) 						\
	static void bfa_ ## __mod ## _meminfo(bfa_ioc_cfg_t *cfg,	\
				uint32_t *ndm_len, uint32_t *dm_len);	\
	static void bfa_ ## __mod ## _attach(bfa_t *bfa,	\
				void *bfad, bfa_ioc_cfg_t *cfg, 	\
				bfa_meminfo_t *meminfo,		\
				bfa_pcidev_t *pcidev);		\
	static void bfa_ ## __mod ## _initdone(bfa_t *bfa);	\
	static void bfa_ ## __mod ## _detach(bfa_t *bfa);	\
	static void bfa_ ## __mod ## _start(bfa_t *bfa);	\
	static void bfa_ ## __mod ## _stop(bfa_t *bfa);		\
	static void bfa_ ## __mod ## _iocdisable(bfa_t *bfa);	\
									\
	extern bfa_module_t hal_mod_ ## __mod;				\
	bfa_module_t hal_mod_ ## __mod = {				\
		bfa_ ## __mod ## _meminfo,				\
		bfa_ ## __mod ## _attach,				\
		bfa_ ## __mod ## _initdone,				\
		bfa_ ## __mod ## _detach,				\
		bfa_ ## __mod ## _start,				\
		bfa_ ## __mod ## _stop,					\
		bfa_ ## __mod ## _iocdisable,				\
	}

#define BFA_CACHELINE_SZ	(256)

/**
 * @brief BFA HAL dma address assignment macro
 */
#define bfa_dma_addr_set(dma_addr, pa)	\
		__bfa_dma_addr_set(&dma_addr, (uint64_t)pa)

static inline void
__bfa_dma_addr_set(bfi_addr_t *dma_addr, uint64_t pa)
{
	dma_addr->a32.addr_lo = (uint32_t) pa;
	dma_addr->a32.addr_hi = (uint32_t) (pa >> 32);
}

/**
 * @brief BFA HAL dma address assignment macro. (big endian format)
 */
#define bfa_dma_be_addr_set(dma_addr, pa)	\
		__bfa_dma_be_addr_set(&dma_addr, (uint64_t)pa)
static inline void
__bfa_dma_be_addr_set(bfi_addr_t *dma_addr, uint64_t pa)
{
	dma_addr->a32.addr_lo = (uint32_t) bfa_os_htonl(pa);
	dma_addr->a32.addr_hi = (uint32_t) bfa_os_htonl(pa >> 32);
}

/**
 * Structure used to interact between different HAL sub modules
 *
 * Each sub module needs to implement only the entry points relevant to it (and
 * can leave entry points as NULL)
 */
typedef struct {
	void	(*meminfo) (bfa_ioc_cfg_t *cfg, uint32_t *km_len,
							uint32_t *dm_len);
	void	(*attach) (bfa_t *bfa, void *bfad,
				   bfa_ioc_cfg_t *cfg,
				   bfa_meminfo_t *meminfo,
				   bfa_pcidev_t *pcidev);
	void	(*initdone) (bfa_t *bfa);
	void	(*detach) (bfa_t *bfa);
	void	(*start) (bfa_t *bfa);
	void	(*stop) (bfa_t *bfa);
	void	(*iocdisable) (bfa_t *bfa);
} bfa_module_t;

extern bfa_module_t *hal_mods[];

struct bfa_s {
	void		*bfad;	/*!< BFA driver instance */
	bfa_aen_t	*aen;	/*!< AEN module */
	bfa_plog_t	*plog;	/*!< portlog buffer */
	bfa_log_mod_t	*logm;	/*!< driver logging module */
	bfa_trc_mod_t	*trcmod;/*!< HAL tracing */
	bfa_pcidev_t *pcidev;
				/*!< PCI device information */
	struct bfa_ioc_s	ioc;	/*!< IOC module */
	bfa_timer_mod_t timer_mod;
				/*!< timer module */
	bfa_modules_t modules;/*!< HAL modules */
	bfa_q_t		comp_q;	/*!< queue of pending completions */
	bfa_q_t         reqq_waitq[BFI_IOC_MAX_CQS];
	bfa_boolean_t   fcs;	/*!< FCS is attached to HAL */
};

#endif /* __HAL_PRIV_H__ */
