/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_AEN_H__
#define __BFA_AEN_H__

#include "defs/bfa_defs_aen.h"

#define BFA_AEN_MAX_ENTRY   512

extern int32_t bfa_aen_max_cfg_entry; 

typedef struct {
	struct bfad_s	*bfad;
	int32_t         max_entry;
	int32_t         write_index;
	int32_t         read_index;
	int32_t         rsvd;					/*!< to force 8 byte alignment of list */
	bfa_aen_entry_t list[BFA_AEN_MAX_ENTRY]; // Must be the last
} bfa_aen_t;

/**
 * Public APIs
 */
static inline void
bfa_aen_set_max_cfg_entry(int max_entry)
{
	bfa_aen_max_cfg_entry = max_entry;
}

static inline int32_t
bfa_aen_get_max_cfg_entry(void)
{
	return bfa_aen_max_cfg_entry;
}

static inline int32_t
bfa_aen_get_meminfo(void)
{
	return (sizeof(bfa_aen_entry_t) * bfa_aen_get_max_cfg_entry());
}

static inline int32_t
bfa_aen_fetch_count(bfa_aen_t *aen)
{
	return ((aen->write_index + aen->max_entry) - aen->read_index)
		% aen->max_entry;
}

int32_t bfa_aen_init(bfa_aen_t *aen, struct bfad_s *bfad);

int32_t bfa_aen_post(bfa_aen_t *aen, bfa_aen_category_t aen_category,
		     bfa_aen_event_t aen_type, bfa_aen_data_t *aen_data);

int32_t bfa_aen_fetch(bfa_aen_t *aen, bfa_aen_entry_t *aen_entry,
		      int32_t entry_space);

int32_t bfa_aen_get_inst(bfa_aen_t *aen);

char *wwn2str(char *buf, int bufsize, uint64_t wwn);

#endif /* __BFA_AEN_H__ */
