/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFI_DIAG_H__
#define __BFI_DIAG_H__

#include <defs/bfa_defs_diag.h>
#include <defs/bfa_defs_pport.h>
#include <protocol/sfp.h>
#include "bfi.h"

#pragma pack(1)

typedef enum {
	BFI_DIAG_H2I_PORTBEACON = 1,
	BFI_DIAG_H2I_LOOPBACK = 2,
	BFI_DIAG_H2I_FWPING = 3,
	BFI_DIAG_H2I_TEMPSENSOR = 4,
	BFI_DIAG_H2I_SFPSHOW = 5,
	BFI_DIAG_H2I_LEDTEST = 6,
	BFI_DIAG_H2I_SFP_SCN	= 7,
} bfi_diag_h2i_t;

typedef enum {
	BFI_DIAG_I2H_PORTBEACON = BFA_I2HM(BFI_DIAG_H2I_PORTBEACON),
	BFI_DIAG_I2H_LOOPBACK = BFA_I2HM(BFI_DIAG_H2I_LOOPBACK),
	BFI_DIAG_I2H_FWPING = BFA_I2HM(BFI_DIAG_H2I_FWPING),
	BFI_DIAG_I2H_TEMPSENSOR = BFA_I2HM(BFI_DIAG_H2I_TEMPSENSOR),
	BFI_DIAG_I2H_SFPSHOW = BFA_I2HM(BFI_DIAG_H2I_SFPSHOW),
	BFI_DIAG_I2H_LEDTEST = BFA_I2HM(BFI_DIAG_H2I_LEDTEST),
	BFI_DIAG_I2H_SFP_SCN	= BFA_I2HM(BFI_DIAG_H2I_SFP_SCN),
} bfi_diag_i2h_t;

#define	BFI_DIAG_MAX_SGES		2
#define	BFI_DIAG_DMA_BUF_SZ		(2 * 1024)

/**
 * Loopback
 */
typedef struct {
	bfi_mhdr_t  mh;
	uint32_t        loopcnt;
	uint32_t        pattern;
	uint8_t         lb_mode;	/*!< bfa_pport_opmode_t */
	uint8_t         speed;		/*!< bfa_pport_speed_t */
	uint8_t         rsvd[2];
} bfi_diag_lb_req_t;

typedef struct {
	bfi_mhdr_t  mh;		/* 4 bytes */
	bfa_diag_loopback_result_t res;	/* 16 bytes */
} bfi_diag_lb_rsp_t;

/**
 * FW Ping
 */
typedef struct {
	bfi_mhdr_t  mh;		/* 4 bytes */
	uint32_t        data;		/*!< user input data pattern */
	bfi_sge_t   sge[BFI_DIAG_MAX_SGES];
	uint32_t        count;		/*!< user input dma count */
	uint8_t         qtag;		/*!< track CPE vc */
	uint8_t		dma_test;	/*!< 1: do dma test */
	uint8_t         rsv[2];
} bfi_diag_fwping_req_t;

typedef struct {
	bfi_mhdr_t  mh;		/* 4 bytes */
	uint32_t        data;		/*!< user input data pattern	*/
	uint8_t         qtag;		/*!< track CPE vc		*/
	uint8_t		dma_status;	/*!< dma status			*/
	uint8_t         rsv[2];
} bfi_diag_fwping_rsp_t;

/**
 * Temperature Sensor
 */
typedef struct {
	bfi_mhdr_t  mh;		/* 4 bytes */
	uint16_t        temp;		/* 10-bit A/D value */
	uint8_t         rsv[2];
} bfi_diag_ts_req_t, bfi_diag_ts_rsp_t;

/**
 * @brief
 * 		SFP memory access type
 */
typedef enum {
	BFI_DIAG_SFP_MEM_NONE = 0x0,
	BFI_DIAG_SFP_MEM_ALL = 0x1,
} bfi_diag_sfpmem_t;

typedef struct {
	bfi_mhdr_t  mh;
	uint8_t         memtype;	/*!< ref bfi_diag_sfpmem_t */
	uint8_t         addr_change;
	uint8_t         static_data;
	uint8_t         rsvd;
	bfi_sge_t       req_sge[BFI_DIAG_MAX_SGES];
} bfi_diag_sfp_req_t;

typedef struct {
	bfi_mhdr_t  mh;
	uint8_t         memtype;	/*!< ref bfi_diag_sfpmem_t */
	uint8_t         status;
	uint8_t         rsvd[2];
} bfi_diag_sfp_rsp_t;

/**
 * LED Test
 */
typedef struct {
	bfi_mhdr_t  mh;	/* 4 bytes */
	uint8_t         cmd;
	uint8_t         color;
	uint8_t         portid;
	uint8_t         led;	/* bitmap of LEDs to be tested */
	uint16_t        freq;	/* no. of blinks every 10 secs */
	uint8_t         rsv[2];
} bfi_diag_ledtest_req_t;

/**
 * Port beaconing
 */
typedef struct {
	bfi_mhdr_t  mh;	/* 4 bytes */
	uint32_t        period;	/*!< beaconing period */
	bfa_boolean_t   beacon;	/*!< 1: beacon on */
} bfi_diag_portbeacon_req_t;

/* notify host the beacon is off */
typedef struct {
	bfi_mhdr_t  mh;	/* 4 bytes */
} bfi_diag_portbeacon_rsp_t;

/**
 *	SFP state notification
 */
typedef struct {
	bfi_mhdr_t	mhr;		/*!< host msg header		*/
	uint8_t		event;
	uint8_t		sfpid;
	uint8_t		pomlvl;		/*!< pom level: normal/warning/alarm*/
	uint8_t		rsvd;
	wwn_t		pwwn;		/*!< pwwn			*/
} bfi_diag_sfp_event_t;

#pragma pack()

#endif /* __BFI_DIAG_H__ */

