/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFI_FLASH_H__
#define __BFI_FLASH_H__

#include "bfi.h"

#pragma pack(1)

typedef enum {
	BFI_FLASH_H2I_QUERY_REQ = 1,
	BFI_FLASH_H2I_ERASE_REQ = 2,
	BFI_FLASH_H2I_WRITE_REQ = 3,
	BFI_FLASH_H2I_READ_REQ = 4,
	BFI_FLASH_H2I_OPTROM_REQ = 5,
} bfi_flash_h2i_msgs_t;

typedef enum {
	BFI_FLASH_I2H_QUERY_RSP = BFA_I2HM(1),
	BFI_FLASH_I2H_ERASE_RSP = BFA_I2HM(2),
	BFI_FLASH_I2H_WRITE_RSP = BFA_I2HM(3),
	BFI_FLASH_I2H_READ_RSP = BFA_I2HM(4),
	BFI_FLASH_I2H_OPTROM_RSP = BFA_I2HM(5),
} bfi_flash_i2h_msgs_t;

/**
 * Flash query request
 */
typedef struct {
	bfi_mhdr_t      mh;		/*!< Common msg header */
	bfi_sge_t       req_sge[BFI_SGE_INLINE_MAX];
} bfi_flash_query_req_t;

/**
 * Flash erase request
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
} bfi_flash_erase_req_t;

/**
 * Flash write request
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        offset;
	uint32_t        length;
	uint32_t        last;
	bfi_sge_t   req_sge[BFI_SGE_INLINE_MAX];
} bfi_flash_write_req_t;

/**
 * Flash read request
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        offset;
	uint32_t        length;
	bfi_sge_t   req_sge[BFI_SGE_INLINE_MAX];
} bfi_flash_read_req_t;

/**
 * Flash option rom enable/disable request
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        optrom_enable;	/*!< option rom enable/disable flag */
} bfi_flash_optrom_req_t;

/**
 * Flash query response
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        status;
} bfi_flash_query_rsp_t;

/**
 * Flash read response
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
	uint32_t        length;
} bfi_flash_read_rsp_t;

/**
 * Flash write response
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
	uint32_t        length;
} bfi_flash_write_rsp_t;

/**
 * Flash erase response
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        type;		/*!< partition type */
	uint32_t        instance;	/*!< partition instance */
	uint32_t        status;
} bfi_flash_erase_rsp_t;

/**
 * Flash option rom enable/disable response
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header */
	uint32_t        status;
} bfi_flash_optrom_rsp_t;

#pragma pack()

#endif /* __BFI_FLASH_H__ */

