/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFI_IOC_H__
#define __BFI_IOC_H__

#include "bfi.h"
#include <defs/bfa_defs_ioc.h>

#pragma pack(1)

typedef enum {
	BFI_IOC_H2I_ENABLE_REQ 		= 1,
	BFI_IOC_H2I_DISABLE_REQ 	= 2,
} bfi_ioc_h2i_msgs_t;

typedef enum {
	BFI_IOC_I2H_ENABLE_REPLY	= BFA_I2HM(1),
	BFI_IOC_I2H_DISABLE_REPLY 	= BFA_I2HM(2),
	BFI_IOC_I2H_READY_EVENT 	= BFA_I2HM(3),
} bfi_ioc_i2h_msgs_t;

/**
 * Firmware trace offset
 */
#define BFI_IOC_TRC_OFF		(0x4b00)
#define BFI_IOC_TRC_ENTS	128

#define BFI_IOC_FW_SIGNATURE	(0xbfadbfad)
typedef struct {
	uint32_t        signature;	/*!< constant signature */
	uint32_t        cksum;		/*!< firmware checksum  */
	uint32_t        exec;		/*!< exec vector        */
	uint32_t        param;		/*!< parameters         */
} bfi_ioc_image_hdr_t;

/**
 *  BFI_IOC_I2H_READY_EVENT message
 */
typedef struct bfa_ioc_rdy_event {
	bfi_mhdr_t  mh;			/*!< common msg header */
	uint8_t         init_status;	/*!< init event status */
	uint8_t         rsvd[3];
} bfi_ioc_rdy_event_t;

/**
 * IOC hardware/firmware state
 */
typedef enum {
	BFI_IOC_UNINIT 	 = 0,		/*!< not initialized                 */
	BFI_IOC_INITING 	 = 1,	/*!< h/w is being initialized        */
	BFI_IOC_HWINIT 	 = 2,		/*!< h/w is initialized              */
	BFI_IOC_CFG 	 = 3,		/*!< IOC configuration in progress   */
	BFI_IOC_OP 		 = 4,	/*!< IOC is operational              */
	BFI_IOC_DISABLING 	 = 5,	/*!< IOC is being disabled           */
	BFI_IOC_DISABLED 	 = 6,	/*!< IOC is disabled                 */
	BFI_IOC_CFG_DISABLED = 7,	/*!< IOC is being disabled;transient */
	BFI_IOC_HBFAIL       = 8,	/*!< IOC heart-beat failure          */
	BFI_IOC_MEMTEST      = 9,	/*!< IOC is doing memtest            */
} bfi_ioc_state_t;

#define BFI_IOC_ENDIAN_SIG  0x12345678

enum {
	BFI_ADAPTER_TYPE_FC   = 0x01,		/*!< FC adapters           */
	BFI_ADAPTER_TYPE_MK   = 0x0f0000,	/*!< adapter type mask     */
	BFI_ADAPTER_TYPE_SH   = 16,	        /*!< adapter type shift    */
	BFI_ADAPTER_NPORTS_MK = 0xff00,		/*!< number of ports mask  */
	BFI_ADAPTER_NPORTS_SH = 8,	        /*!< number of ports shift */
	BFI_ADAPTER_SPEED_MK  = 0xff,		/*!< adapter speed mask    */
	BFI_ADAPTER_SPEED_SH  = 0,	        /*!< adapter speed shift   */
	BFI_ADAPTER_PROTO     = 0x100000,	/*!< prototype adapaters   */
	BFI_ADAPTER_TTV       = 0x200000,	/*!< TTV debug capable     */
	BFI_ADAPTER_UNSUPP    = 0x400000,	/*!< unknown adapter type  */
};

#define BFI_ADAPTER_GETP(__prop,__adap_prop)          		\
    (((__adap_prop) & BFI_ADAPTER_ ## __prop ## _MK) >>         \
     BFI_ADAPTER_ ## __prop ## _SH)
#define BFI_ADAPTER_SETP(__prop, __val)         		\
    ((__val) << BFI_ADAPTER_ ## __prop ## _SH)
#define BFI_ADAPTER_IS_PROTO(__adap_type)   			\
    ((__adap_type) & BFI_ADAPTER_PROTO)
#define BFI_ADAPTER_IS_TTV(__adap_type)     			\
    ((__adap_type) & BFI_ADAPTER_TTV)
#define BFI_ADAPTER_IS_UNSUPP(__adap_type)  			\
    ((__adap_type) & BFI_ADAPTER_UNSUPP)
#define BFI_ADAPTER_IS_SPECIAL(__adap_type)                     \
    ((__adap_type) & (BFI_ADAPTER_TTV | BFI_ADAPTER_PROTO |     \
			BFI_ADAPTER_UNSUPP))

/**
 * BFI_IOC_H2I_ENABLE_REQ & BFI_IOC_H2I_DISABLE_REQ messages
 */
typedef struct {
	bfi_mhdr_t  mh;
} bfi_ioc_enable_req_t, bfi_ioc_disable_req_t;

/**
 * BFI_IOC_I2H_ENABLE_REPLY & BFI_IOC_I2H_DISABLE_REPLY messages
 */
typedef struct {
	bfi_mhdr_t  mh;			/*!< Common msg header     */
	uint8_t         status;		/*!< enable/disable status */
	uint8_t         rsvd[3];
} bfi_ioc_enable_reply_t, bfi_ioc_disable_reply_t;

#define BFI_IOC_MSGSZ   8
/**
 * H2I Messages
 */
typedef union {
	bfi_mhdr_t		mh;
	bfi_ioc_enable_req_t	enable_req;
	bfi_ioc_disable_req_t	disable_req;
	uint32_t       		mboxmsg[BFI_IOC_MSGSZ];
} bfi_ioc_h2i_msg_t;

/**
 * I2H Messages
 */
typedef union {
	bfi_mhdr_t      	mh;
	bfi_ioc_rdy_event_t 	rdy_event;
	uint32_t       		mboxmsg[BFI_IOC_MSGSZ];
} bfi_ioc_i2h_msg_t;

#pragma pack()

#endif /* __BFI_IOC_H__ */

