/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFI_IOCFC_H__
#define __BFI_IOCFC_H__

#include "bfi.h"
#include <defs/bfa_defs_ioc.h>

#pragma pack(1)

typedef enum {
	BFI_IOCFC_H2I_CFG_REQ 		= 1,
	BFI_IOCFC_H2I_GET_STATS_REQ 	= 2,
	BFI_IOCFC_H2I_CLEAR_STATS_REQ	= 3,
	BFI_IOCFC_H2I_SET_INTR_REQ 	= 4,
} bfi_iocfc_h2i_msgs_t;

typedef enum {
	BFI_IOCFC_I2H_CFG_REPLY		= BFA_I2HM(1),
	BFI_IOCFC_I2H_GET_STATS_RSP 	= BFA_I2HM(2),
	BFI_IOCFC_I2H_CLEAR_STATS_RSP	= BFA_I2HM(3),
} bfi_iocfc_i2h_msgs_t;

typedef struct {
	uint8_t         num_cqs; 	/*!< Number of CQs to be used     */
	uint8_t         sense_buf_len;	/*!< SCSI sense length            */
	uint8_t         trunk_enabled;	/*!< port trunking enabled        */
	uint8_t         trunk_ports;	/*!< trunk ports bit map          */
	uint32_t        endian_sig;	/*!< endian signature of host     */

	/**
	 * Request and response circular queue base addresses, size and
	 * shadow index pointers.
	 */
	bfi_addr_t  req_cq_ba[BFI_IOC_MAX_CQS];
	bfi_addr_t  req_shadow_ci[BFI_IOC_MAX_CQS];
	uint16_t    req_cq_elems[BFI_IOC_MAX_CQS];
	bfi_addr_t  rsp_cq_ba[BFI_IOC_MAX_CQS];
	bfi_addr_t  rsp_shadow_pi[BFI_IOC_MAX_CQS];
	uint16_t    rsp_cq_elems[BFI_IOC_MAX_CQS];

	bfi_addr_t  stats_addr;		/*!< DMA-able address for stats	  */
	bfi_addr_t  cfgrsp_addr;	/*!< config response dma address  */
	bfi_addr_t  ioim_snsbase;   	/*!< IO sense buffer base address */
	bfa_ioc_intr_attr_t intr_attr;	/*!< IOC interrupt attributes     */
} bfi_iocfc_cfg_t;

typedef struct {
	wwn_t           mfg_wwn;
	char            brcd_serialnum[STRSZ(BFA_MFG_SERIALNUM_SIZE)];
	uint8_t         pcie_gen;
	uint8_t         pcie_lanes_orig;
	uint8_t         pcie_lanes;
	uint8_t         rx_bbcredit;	/*!< receive buffer credits */
	uint32_t        adapter_prop;	/*!< adapter properties     */
	uint16_t        maxfrsize;	/*!< max receive frame size */
	char         	asic_rev;
	uint8_t         rsvd;
	bfa_ioc_fwcfg_t fwcfg;	        /*!< firmware configuration */
	bfa_ioc_intr_attr_t intr_attr;	/*!< interrupt attributes   */
	char            fw_version[BFA_VERSION_LEN];
	char            optrom_version[BFA_VERSION_LEN];
} bfi_iocfc_cfgrsp_t;

/**
 * BFI_IOCFC_H2I_CFG_REQ message
 */
typedef struct {
	bfi_mhdr_t      mh;
	bfi_addr_t      ioc_cfg_dma_addr;
} bfi_iocfc_cfg_req_t;

/**
 * BFI_IOCFC_I2H_CFG_REPLY message
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< Common msg header          */
	uint8_t         cfg_success;	/*!< cfg reply status           */
	uint8_t         lpu_bm;		/*!< LPUs assigned for this IOC */
	uint8_t         rsvd[2];
} bfi_iocfc_cfg_reply_t;

/**
 *  BFI_IOCFC_H2I_GET_STATS_REQ & BFI_IOCFC_H2I_CLEAR_STATS_REQ messages
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< msg header            */
	uint32_t        msgtag;		/*!< msgtag for reply      */
} bfi_iocfc_stats_get_req_t, bfi_iocfc_stats_clr_req_t;

/**
 * BFI_IOCFC_I2H_GET_STATS_RSP & BFI_IOCFC_I2H_CLEAR_STATS_RSP messages
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header     */
	uint8_t         status;		/*!< reply status          */
	uint8_t         rsvd[3];
	uint32_t        msgtag;		/*!< msgtag for reply      */
} bfi_iocfc_stats_get_rsp_t, bfi_iocfc_stats_clr_rsp_t;

/**
 * BFI_IOCFC_H2I_SET_INTR_REQ message
 */
typedef struct {
	bfi_mhdr_t	mh;		/*!< common msg header     */
	uint8_t		coalesce;	/*!< enable intr coalescing*/
	uint8_t         rsvd[3];
	uint16_t	delay;		/*!< delay timer 0..1125us  */
	uint16_t	latency;	/*!< latency timer 0..225us */
} bfi_iocfc_set_intr_req_t;

/**
 * H2I Messages
 */
typedef union {
	bfi_mhdr_t			mh;
	bfi_iocfc_cfg_req_t		cfg_req;
	bfi_iocfc_stats_get_req_t	stats_get;
	bfi_iocfc_stats_clr_req_t	stats_clr;
	uint32_t       			mboxmsg[BFI_IOC_MSGSZ];
} bfi_iocfc_h2i_msg_t;

/**
 * I2H Messages
 */
typedef union {
	bfi_mhdr_t      		mh;
	bfi_iocfc_cfg_reply_t 		cfg_reply;
	bfi_iocfc_stats_get_rsp_t	stats_get_rsp;
	bfi_iocfc_stats_clr_rsp_t	stats_clr_rsp;
	uint32_t       			mboxmsg[BFI_IOC_MSGSZ];
} bfi_iocfc_i2h_msg_t;

#pragma pack()

#endif /* __BFI_IOCFC_H__ */

