/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFI_PPORT_H__
#define __BFI_PPORT_H__

#include <bfi/bfi.h>
#include <defs/bfa_defs_pport.h>

#pragma pack(1)

typedef enum {
	BFI_PPORT_H2I_ENABLE_REQ		= (1),
	BFI_PPORT_H2I_DISABLE_REQ		= (2),
	BFI_PPORT_H2I_GET_STATS_REQ		= (3),
	BFI_PPORT_H2I_CLEAR_STATS_REQ		= (4),
	BFI_PPORT_H2I_SET_SVC_PARAMS_REQ	= (5),
	BFI_PPORT_H2I_ENABLE_RX_VF_TAG_REQ	= (6),
	BFI_PPORT_H2I_ENABLE_TX_VF_TAG_REQ	= (7),
	BFI_PPORT_H2I_GET_QOS_STATS_REQ		= (8),
	BFI_PPORT_H2I_CLEAR_QOS_STATS_REQ	= (9),
} bfi_pport_h2i_t;

typedef enum {
	BFI_PPORT_I2H_ENABLE_RSP		= BFA_I2HM(1),
	BFI_PPORT_I2H_DISABLE_RSP		= BFA_I2HM(2),
	BFI_PPORT_I2H_GET_STATS_RSP		= BFA_I2HM(3),
	BFI_PPORT_I2H_CLEAR_STATS_RSP		= BFA_I2HM(4),
	BFI_PPORT_I2H_SET_SVC_PARAMS_RSP	= BFA_I2HM(5),
	BFI_PPORT_I2H_ENABLE_RX_VF_TAG_RSP	= BFA_I2HM(6),
	BFI_PPORT_I2H_ENABLE_TX_VF_TAG_RSP	= BFA_I2HM(7),
	BFI_PPORT_I2H_EVENT			= BFA_I2HM(8),
	BFI_PPORT_I2H_GET_QOS_STATS_RSP		= BFA_I2HM(9),
	BFI_PPORT_I2H_CLEAR_QOS_STATS_RSP	= BFA_I2HM(10),
} bfi_pport_i2h_t;

/**
 * Generic REQ type
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< msg header			    */
	uint32_t        msgtag;		/*!< msgtag for reply		    */
} bfi_pport_generic_req_t;

/**
 * Generic RSP type
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< common msg header		    */
	uint8_t         status;		/*!< port enable status		    */
	uint8_t         rsvd[3];
	uint32_t        msgtag;		/*!< msgtag for reply		    */
} bfi_pport_generic_rsp_t;

/**
 * BFI_PPORT_H2I_ENABLE_REQ
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< msg header			    */
	uint32_t        rsvd1;
	wwn_t           nwwn;		/*!< node wwn of physical port	    */
	wwn_t           pwwn;		/*!< port wwn of physical port	    */
	bfa_pport_cfg_t port_cfg;	/*!< port configuration		    */
	bfi_addr_t  stats_dma_addr;	/*!< DMA address for stats	    */
	uint32_t        msgtag;		/*!< msgtag for reply		    */
	uint32_t        rsvd2;
} bfi_pport_enable_req_t;

/**
 * BFI_PPORT_I2H_ENABLE_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_enable_rsp_t;

/**
 * BFI_PPORT_H2I_DISABLE_REQ
 */
typedef bfi_pport_generic_req_t bfi_pport_disable_req_t;

/**
 * BFI_PPORT_I2H_DISABLE_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_disable_rsp_t;

/**
 * BFI_PPORT_H2I_GET_STATS_REQ
 */
typedef bfi_pport_generic_req_t bfi_pport_get_stats_req_t;

/**
 * BFI_PPORT_I2H_GET_STATS_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_get_stats_rsp_t;

/**
 * BFI_PPORT_H2I_CLEAR_STATS_REQ
 */
typedef bfi_pport_generic_req_t bfi_pport_clear_stats_req_t;

/**
 * BFI_PPORT_I2H_CLEAR_STATS_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_clear_stats_rsp_t;

/**
 * BFI_PPORT_H2I_GET_QOS_STATS_REQ
 */
typedef bfi_pport_generic_req_t bfi_pport_get_qos_stats_req_t;

/**
 * BFI_PPORT_H2I_GET_QOS_STATS_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_get_qos_stats_rsp_t;

/**
 * BFI_PPORT_H2I_CLEAR_QOS_STATS_REQ
 */
typedef bfi_pport_generic_req_t bfi_pport_clear_qos_stats_req_t;

/**
 * BFI_PPORT_H2I_CLEAR_QOS_STATS_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_clear_qos_stats_rsp_t;

/**
 * BFI_PPORT_H2I_SET_SVC_PARAMS_REQ
 */
typedef struct {
	bfi_mhdr_t  mh;		/*!< msg header */
	uint16_t        tx_bbcredit;	/*!< Tx credits */
	uint16_t        rsvd;
} bfi_pport_set_svc_params_req_t;

/**
 * BFI_PPORT_I2H_SET_SVC_PARAMS_RSP
 */
typedef bfi_pport_generic_rsp_t bfi_pport_set_svc_params_rsp_t;

/**
 * BFI_PPORT_I2H_EVENT
 */
typedef struct {
	bfi_mhdr_t		mh;	/*!< common msg header */
	bfa_pport_link_t	link_state;
} bfi_pport_event_t;

typedef union {
	bfi_mhdr_t			*mhdr;
	bfi_pport_enable_req_t		*penable;
	bfi_pport_disable_req_t		*pdisable;
	bfi_pport_get_stats_req_t	*pgetstats;
	bfi_pport_get_qos_stats_req_t	*pgetqosstats;
	bfi_pport_clear_stats_req_t	*pclearstats;
	bfi_pport_clear_qos_stats_req_t	*pclearqosstats;
	bfi_pport_set_svc_params_req_t	*psetsvcparams;
} bfi_pport_h2i_msg_t;

typedef union {
	bfi_msg_t			*msg;
	bfi_pport_enable_rsp_t		*enable_rsp;
	bfi_pport_disable_rsp_t		*disable_rsp;
	bfi_pport_get_stats_rsp_t	*getstats_rsp;
	bfi_pport_get_qos_stats_rsp_t	*getqosstats_rsp;
	bfi_pport_clear_stats_rsp_t	*clearstats_rsp;
	bfi_pport_clear_qos_stats_rsp_t	*clearqosstats_rsp;
	bfi_pport_set_svc_params_rsp_t	*setsvcparasm_rsp;
	bfi_pport_event_t		*event;
} bfi_pport_i2h_msg_t;

#pragma pack()

#endif /* __BFI_PPORT_H__ */

