/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_BOOT_H__
#define __BFA_DEFS_BOOT_H__

#include <protocol/types.h>
#include <defs/bfa_defs_types.h>
#include <defs/bfa_defs_pport.h>

enum {
	BFA_BOOT_BOOTLUN_MAX = 4,	/*!< maximum boot lun per IOC */
};

#define BOOT_CFG_REV1	1

/** 
 * @brief
 *      Boot options setting. Boot options setting determines from where
 *      to get the boot lun information
 */ 
typedef enum {
    BFA_BOOT_AUTO_DISCOVER = 0,    /*!< Boot from blun provided by fabric */
    BFA_BOOT_STORED_BLUN   = 1,    /*!< Boot from bluns stored in flash   */
    BFA_BOOT_FIRST_LUN     = 2,    /*!< Boot from first discovered blun   */
} bfa_boot_bootopt_t;

/**
 * Boot lun information.
 */
typedef struct {
	wwn_t           pwwn;	/*!< port wwn of target */
	lun_t           lun;	/*!< 64-bit lun */
} bfa_boot_bootlun_t;

/**
 * BOOT boot configuraton
 */
typedef struct {
	uint8_t         version;
	uint8_t         rsvd1;
	uint16_t        chksum;

	uint8_t         enable;		/*!< enable/disable SAN boot */
	uint8_t         speed;		/*!< boot speed settings */
	uint8_t         topology;	/*!< boot topology setting */
	uint8_t         bootopt;	/*!< bfa_boot_bootopt_t */

	uint32_t        nbluns;		/*!< number of boot luns */

	uint32_t        rsvd2;

	bfa_boot_bootlun_t blun[BFA_BOOT_BOOTLUN_MAX];
} bfa_boot_cfg_t;

#endif /* __BFA_DEFS_BOOT_H__ */
