/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_ITNIM_H__
#define __BFA_DEFS_ITNIM_H__

#include <bfa_os_inc.h>
#include <protocol/types.h>

/**
 * FCS itnim states
 */
typedef enum {
	BFA_ITNIM_OFFLINE 	= 0,	/*!< offline */
	BFA_ITNIM_PRLI_SEND 	= 1,	/*!< prli send */
	BFA_ITNIM_PRLI_SENT 	= 2,	/*!< prli sent */
	BFA_ITNIM_PRLI_RETRY 	= 3,	/*!< prli retry */
	BFA_ITNIM_HCB_ONLINE 	= 4,	/*!< online callback */
	BFA_ITNIM_ONLINE 	= 5,	/*!< online */
	BFA_ITNIM_HCB_OFFLINE 	= 6,	/*!< offline callback */
	BFA_ITNIM_INITIATIOR 	= 7,	/*!< initiator */
} bfa_itnim_state_t;

typedef struct {
	uint32_t	onlines;	/*!< ITN nexus onlines (PRLI done) */
	uint32_t	offlines;	/*!< ITN Nexus offlines 	*/
	uint32_t	creates;	/*!< ITN create requests 	*/
	uint32_t	deletes;	/*!< ITN delete requests 	*/
	uint32_t	create_comps;	/*!< ITN create completions 	*/
	uint32_t	delete_comps;	/*!< ITN delete completions 	*/
	uint32_t	sler_events;	/*!< SLER (sequence level error
					 * recovery) events */
	uint32_t	ioc_disabled;	/*!< Num IOC disables		*/
	uint32_t	cleanup_comps;	/*!< ITN cleanup completions */
	uint32_t	tm_cmnds;	/*!< task management(TM) cmnds sent */
	uint32_t	tm_fw_rsps;	/*!< TM cmds firmware responses */
	uint32_t	tm_success;	/*!< TM successes */
	uint32_t	tm_failures;	/*!< TM failures */
	uint32_t	tm_io_comps;	/*!< TM IO completions */
	uint32_t	tm_qresumes;	/*!< TM queue resumes (after waiting
					 * for resources)
					 */
	uint32_t	tm_iocdowns;	/*!< TM cmnds affected by IOC down */
	uint32_t	tm_cleanups;	/*!< TM cleanups */
	uint32_t	tm_cleanup_comps;
					/*!< TM cleanup completions */
	uint32_t	ios;		/*!< IO requests */
	uint32_t	io_comps;	/*!< IO completions */
} bfa_itnim_hal_stats_t;

/**
 * FCS remote port statistics
 */
typedef struct {
	uint32_t        onlines;	/*!< num rport online */
	uint32_t        offlines;	/*!< num rport offline */
	uint32_t        prli_sent;	/*!< num prli sent out */
	uint32_t        fcxp_alloc_wait;/*!< num fcxp alloc waits */
	uint32_t        prli_rsp_err;	/*!< num prli rsp errors */
	uint32_t        prli_rsp_acc;	/*!< num prli rsp accepts */
	uint32_t        initiator;	/*!< rport is an initiator */
	uint32_t        prli_rsp_parse_err;	/*!< prli rsp parsing errors */
	uint32_t        prli_rsp_rjt;	/*!< num prli rsp rejects */
	uint32_t        timeout;	/*!< num timeouts detected */
	uint32_t        sler;		/*!< num sler notification from HAL */
	uint32_t	rsvd;
	bfa_itnim_hal_stats_t	hal_stats;
} bfa_itnim_stats_t;

/**
 * FCS itnim attributes returned in queries
 */
typedef struct {
	bfa_itnim_state_t	state;          /*!< FCS itnim state            */
	uint8_t        		retry;		/*!< data retransmision support */
	uint8_t			task_retry_id;  /*!< task retry ident support   */
	uint8_t        	    	rec_support;    /*!< REC supported              */
	uint8_t        	    	conf_comp;      /*!< confirmed completion supp  */
} bfa_itnim_attr_t;

/**
 * BFA ITNIM events.
 * Arguments below are in BFAL context from Mgmt
 * BFA_ITNIM_AEN_NEW:       [in]: None  [out]: vf_id, lpwwn
 * BFA_ITNIM_AEN_DELETE:    [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, ppwwn, lpwwn, rpwwn
 * BFA_ITNIM_AEN_ONLINE:    [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, ppwwn, lpwwn, rpwwn
 * BFA_ITNIM_AEN_OFFLINE:   [in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, ppwwn, lpwwn, rpwwn
 * BFA_ITNIM_AEN_DISCONNECT:[in]: vf_id, lpwwn, rpwwn (0 = all fcp4 targets),
 *				  [out]: vf_id, ppwwn, lpwwn, rpwwn
 */
typedef enum {
	BFA_ITNIM_AEN_ONLINE 	= 1,	/*!< Target online */
	BFA_ITNIM_AEN_OFFLINE 	= 2,	/*!< Target offline */
	BFA_ITNIM_AEN_DISCONNECT = 3,	/*!< Target disconnected */
} bfa_itnim_aen_event_t;

/**
 * BFA ITNIM event data structure.
 */
typedef struct {
	uint16_t        vf_id;	/*!< vf_id of the IT nexus */
	uint16_t        rsvd[3];
	wwn_t           ppwwn;	/*!< WWN of its physical port */
	wwn_t           lpwwn;	/*!< WWN of logical port */
	wwn_t           rpwwn;	/*!< WWN of remote(target) port */
} bfa_itnim_aen_data_t;
#endif /* __BFA_DEFS_ITNIM_H__ */
