/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_QOS_H__
#define __BFA_DEFS_QOS_H__

/**
 * QoS states
 */
typedef enum {
	BFA_QOS_ONLINE = 1,		/*!< QoS is online */
	BFA_QOS_OFFLINE = 2,		/*!< QoS is offline */
} bfa_qos_state_t;


/**
 * QoS  Priority levels.
 */
typedef enum {
	BFA_QOS_UNKNOWN = 0,
	BFA_QOS_HIGH  = 1,	/*!< QoS Priority Level High */
	BFA_QOS_MED  =  2,	/*!< QoS Priority Level Medium */
	BFA_QOS_LOW  =  3,	/*!< QoS Priority Level Low */
} bfa_qos_priority_t;


/**
 * QoS  bandwidth allocation for each priority level
 */
typedef enum {
	BFA_QOS_BW_HIGH  = 60,	/*!< bandwidth allocation for High */
	BFA_QOS_BW_MED  =  30,	/*!< bandwidth allocation for Medium */
	BFA_QOS_BW_LOW  =  10,	/*!< bandwidth allocation for Low */
} bfa_qos_bw_alloc_t;

/**
 * QoS attribute returned in QoS Query
 */
typedef struct {
	bfa_qos_state_t state;			/*!< QoS current state */
	uint32_t  total_bb_cr;  	 	/*!< Total BB Credits */
} bfa_qos_attr_t;

/**
 * These fields should be displayed only from the CLI.
 * There will be a separate BFAL API (get_qos_vc_attr ?)
 * to retrieve this.
 *
 */
#define  BFA_QOS_MAX_VC  16

typedef struct {
	uint8_t vc_credit;
	uint8_t borrow_credit;
	uint8_t priority;
	uint8_t resvd;
} bfa_qos_vc_info_t;

typedef struct {
	uint16_t  total_vc_count;                    /*!< Total VC Count */
	uint16_t   shared_credit;
	uint32_t   elp_opmode_flags;
	bfa_qos_vc_info_t vc_info[BFA_QOS_MAX_VC];  /*!<  as many as
						     				     *    total_vc_count */
} bfa_qos_vc_attr_t;

/**
 * QoS statistics
 */
typedef struct {
	uint32_t	flogi_sent; 		/*!< QoS Flogi sent */
	uint32_t	flogi_acc_recvd;	/*!< QoS Flogi Acc received */
	uint32_t	flogi_rjt_recvd;	/*!< QoS Flogi rejects received */
	uint32_t	flogi_retries;		/*!< QoS Flogi retries */

	uint32_t	elp_recvd; 	   		/*!< QoS ELP received */
	uint32_t	elp_accepted;       /*!< QoS ELP Accepted */
	uint32_t	elp_rejected;       /*!< QoS ELP rejected */
	uint32_t	elp_dropped;        /*!< QoS ELP dropped  */

	uint32_t	qos_rscn_recvd;     /*!< QoS RSCN received */
	uint32_t	rsvd; 				/* padding for 64 bit alignment */
} bfa_qos_stats_t;


#endif /* __BFA_DEFS_QOS_H__ */
