/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_DEFS_STATUS_H__
#define __BFA_DEFS_STATUS_H__

/**
 * API status return values
 *
 * NOTE: The error msgs are auto generated from the comments. Only singe line
 * comments are supported
 */
typedef enum {
	BFA_STATUS_OK 		= 0,	/*!< Success */
	BFA_STATUS_FAILED 	= 1,	/*!< Operation failed */
	BFA_STATUS_EINVAL 	= 2,	/*!< Invalid params - Check input parameters */
	BFA_STATUS_ENOMEM 	= 3,	/*!< Out of resources */
	BFA_STATUS_ENOSYS 	= 4,	/*!< Function not implemented */
	BFA_STATUS_ETIMER 	= 5,	/*!< Timer expired - Retry and if it persists, contact support */
	BFA_STATUS_EPROTOCOL 	= 6,	/*!< Protocol error */
	BFA_STATUS_ENOFCPORTS 	= 7,	/*!< No FC ports resources */
	BFA_STATUS_NOFLASH 	= 8,	/*!< Flash not present */
	BFA_STATUS_BADFLASH 	= 9,	/*!< Flash is corrupted or bad */
	BFA_STATUS_SFP_UNSUPP 	= 10,	/*!< Unsupported SFP - Replace SFP */
	BFA_STATUS_UNKNOWN_VFID = 11,	/*!< VF_ID not found */
	BFA_STATUS_DATACORRUPTED = 12,	/*!< Diag returned data corrupted - Contact support */
	BFA_STATUS_DEVBUSY 	= 13,	/*!< Device busy - Retry operation */
	BFA_STATUS_ABORTED 	= 14,	/*!< Operation aborted */
	BFA_STATUS_NODEV 	= 15,	/*!< Dev is not present */
	BFA_STATUS_HDMA_FAILED 	= 16,	/*!< Host dma failed - Contact support */
	BFA_STATUS_FLASH_BAD_LEN = 17,	/*!< Flash bad length */
	BFA_STATUS_UNKNOWN_LWWN = 18,	/*!< LPORT PWWN not found */
	BFA_STATUS_UNKNOWN_RWWN = 19,	/*!< RPORT PWWN not found */
	BFA_STATUS_FCPT_LS_RJT 	= 20,	/*!< Got LS_RJT for FC Pass through Req */
	BFA_STATUS_VPORT_EXISTS = 21,	/*!< VPORT already exists */
	BFA_STATUS_VPORT_MAX 	= 22,	/*!< Reached max VPORT supported limit
					 */
	BFA_STATUS_UNSUPP_SPEED = 23,	/*!< Invalid Speed - Check speed setting */
	BFA_STATUS_INVLD_DFSZ 	= 24,	/*!< Invalid Max data field size */
	BFA_STATUS_CNFG_FAILED 	= 25,	/*!< Setting can not be persisted */
	BFA_STATUS_CMD_NOTSUPP 	= 26,	/*!< Command/API not supported */
	BFA_STATUS_NO_ADAPTER 	= 27,	/*!< No Brocade HBA Found */
	BFA_STATUS_LINKDOWN 	= 28,	/*!< Link is down - Check or replace SFP/cable */
	BFA_STATUS_FABRIC_RJT 	= 29,	/*!< Reject from attached fabric */
	BFA_STATUS_UNKNOWN_VWWN = 30,	/*!< VPORT PWWN not found */
	BFA_STATUS_NSLOGIN_FAILED = 31,	/*!< Nameserver login failed */
	BFA_STATUS_NO_RPORTS 	= 32,	/*!< No remote ports found */
	BFA_STATUS_NSQUERY_FAILED = 33,	/*!< Nameserver query failed */
	BFA_STATUS_PORT_OFFLINE = 34,	/*!< Port is not online */
	BFA_STATUS_RPORT_OFFLINE = 35,	/*!< RPORT is not online */
	BFA_STATUS_TGTOPEN_FAILED = 36,	/*!< Remote SCSI target open failed */
	BFA_STATUS_BAD_LUNS 	= 37,	/*!< No valid LUNs found */
	BFA_STATUS_IO_FAILURE 	= 38,	/*!< SCSI target IO failure */
	BFA_STATUS_NO_FABRIC 	= 39,	/*!< No switched fabric present */
	BFA_STATUS_EBADF 	= 40,	/*!< Bad file descriptor */
	BFA_STATUS_EINTR 	= 41,	/*!< A signal was caught during ioctl */
	BFA_STATUS_EIO 		= 42,	/*!< I/O error */
	BFA_STATUS_ENOTTY 	= 43,	/*!< Inappropriate I/O control operation */
	BFA_STATUS_ENXIO 	= 44,	/*!< No such device or address */
	BFA_STATUS_EFOPEN 	= 45,	/*!< Failed to open file */
	BFA_STATUS_VPORT_WWN_BP = 46,	/*!< WWN is same as base port's WWN */
	BFA_STATUS_PORT_NOT_DISABLED = 47, /*!< Port not disabled - Disable port first */
	BFA_STATUS_BADFRMHDR 	= 48,	/*!< Bad frame header */
	BFA_STATUS_BADFRMSZ 	= 49,	/*!< Bad frame size - Check and replace SFP/cable */
	BFA_STATUS_MISSINGFRM 	= 50,	/*!< Missing frame - Check and replace SFP/cable */
	BFA_STATUS_LINKTIMEOUT 	= 51,	/*!< Link timeout - Check and replace SFP/cable */
	BFA_STATUS_NO_FCPIM_NEXUS = 52,	/*!< No FCP Nexus exists with the  rport */
	BFA_STATUS_CHECKSUM_FAIL = 53,	/*!< checksum failure */
	BFA_STATUS_GZME_FAILED 	= 54,	/*!< Get zone member query failed */
	BFA_STATUS_SCSISTART_REQD = 55,	/*!< SCSI disk require START command */
	BFA_STATUS_IOC_FAILURE 	= 56,	/*!< IOC failure - Retry and if it still fails, contact support */
	BFA_STATUS_INVALID_WWN 	= 57,	/*!< Invalid WWN */
	BFA_STATUS_MISMATCH 	= 58,	/*!< Version mismatch */
	BFA_STATUS_IOC_ENABLED 	= 59,	/*!< IOC is already enabled */
	BFA_STATUS_ADAPTER_ENABLED = 60, /*!< Adapter is not disabled - Disable adapter first */
	BFA_STATUS_IOC_NON_OP 	= 61,	/*!< IOC is not operational - Enable ioc and if it still fails, contact support */
	BFA_STATUS_ADDR_MAP_FAILURE = 62, /*!< PCI base address not mapped in OS */
	BFA_STATUS_SAME_NAME 	= 63,	/*!< Name exists! use a different name */
	BFA_STATUS_PENDING      = 64,   /*!< API completes asynchronously */
	BFA_STATUS_8G_SPD	= 65,	/*!< Speed setting not valid for 8G HBA */
	BFA_STATUS_4G_SPD	= 66,	/*!< Speed setting not valid for 4G HBA */
	BFA_STATUS_AD_IS_ENABLE = 67,	/*!< Adapter is already enabled */
	BFA_STATUS_EINVAL_TOV 	= 68,	/*!< Invalid path failover TOV */
	BFA_STATUS_EINVAL_QDEPTH = 69,	/*!< Invalid queue depth value */
	BFA_STATUS_VERSION_FAIL = 70,	/*!< Application/Driver version mismatch */
	BFA_STATUS_DIAG_BUSY    = 71,	/*!< diag busy */
	BFA_STATUS_BEACON_ON	= 72,	/*!< Port Beacon already on */
	BFA_STATUS_BEACON_OFF	= 73,	/*!< Port Beacon already off */
	BFA_STATUS_LBEACON_ON   = 74,	/*!< Link End-to-End Beacon already on */
	BFA_STATUS_LBEACON_OFF	= 75,	/*!< Link End-to-End Beacon already off */
	BFA_STATUS_PORT_NOT_INITED = 76, /*!< Port not initialized */
	BFA_STATUS_RPSC_ENABLED = 77, /*!< Target has a valid speed */
	BFA_STATUS_ENOFSAVE 	= 78,	/*!< No saved firmware trace */
	BFA_STATUS_BAD_FILE		= 79,	/*!< Not a valid Brocade Boot Code file */
	BFA_STATUS_RLIM_EN		= 80,	/*!< Target rate limiting is already enabled */
	BFA_STATUS_RLIM_DIS		= 81,  /*!< Target rate limiting is already disabled */
	BFA_STATUS_IOC_DISABLED  = 82,   /*!< IOC is already disabled */
	BFA_STATUS_ADAPTER_DISABLED  = 83,   /*!< Adapter is already disabled */
	BFA_STATUS_BIOS_DISABLED  = 84,   /*!< Bios is already disabled */
	BFA_STATUS_AUTH_ENABLED  = 85,   /*!< Authentication is already enabled */
	BFA_STATUS_AUTH_DISABLED  = 86,   /*!< Authentication is already disabled */
	BFA_STATUS_ERROR_TRL_ENABLED  = 87,   /*!< Target rate limiting is enabled */
	BFA_STATUS_ERROR_QOS_ENABLED  = 88,   /*!< QoS is enabled */
	BFA_STATUS_NO_SFP_DEV = 89,     /*!< No SFP device - Check or replace SFP */
	BFA_STATUS_MEMTEST_FAILED = 90,	/*!< Memory test failed - Contact support */
	BFA_STATUS_INVALID_DEVID = 91,	/*!< Invalid device id provided */
	BFA_STATUS_QOS_ENABLED = 92, /*!< QOS is already enabled */
	BFA_STATUS_QOS_DISABLED = 93, /*!< QOS is already disabled */
	BFA_STATUS_MAX_VAL		/*!< Unknown error code */
} bfa_status_t;

#endif /* __BFA_DEFS_STATUS_H__ */
