/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_DEFS_VF_H__
#define __BFA_DEFS_VF_H__

#include <bfa_os_inc.h>
#include <defs/bfa_defs_port.h>
#include <protocol/types.h>

/**
 * VF states
 */
typedef enum {
	BFA_VF_UNINIT    = 0,	/*!< fabric is not yet initialized */
	BFA_VF_LINK_DOWN = 1,	/*!< link is down */
	BFA_VF_FLOGI     = 2,	/*!< flogi is in progress */
	BFA_VF_AUTH      = 3,	/*!< authentication in progress */
	BFA_VF_NOFABRIC  = 4,	/*!< fabric is not present */
	BFA_VF_ONLINE    = 5,	/*!< login to fabric is complete */
	BFA_VF_EVFP      = 6,	/*!< EVFP is in progress */
	BFA_VF_ISOLATED  = 7,	/*!< port isolated due to vf_id mismatch */
} bfa_vf_state_t;

/**
 * VF statistics
 */
typedef struct {
	uint32_t        flogi_sent;		/*!< Num FLOGIs sent 		*/
	uint32_t        flogi_rsp_err;		/*!< FLOGI response errors 	*/
	uint32_t        flogi_acc_err;		/*!< FLOGI accept errors 	*/
	uint32_t        flogi_accepts;		/*!< FLOGI accepts received 	*/
	uint32_t        flogi_rejects;		/*!< FLOGI rejects received 	*/
	uint32_t        flogi_unknown_rsp;	/*!< Unknown responses for FLOGI */
	uint32_t        flogi_alloc_wait;	/*!< Allocation waits prior to
						 * sending FLOGI
						 */
	uint32_t        flogi_rcvd;		/*!< FLOGIs received */
	uint32_t        flogi_rejected;		/*!< Incoming FLOGIs rejected */
	uint32_t        fabric_onlines;		/*!< Internal fabric online
						 * notification sent to other
						 * modules
						 */
	uint32_t        fabric_offlines;	/*!< Internal fabric offline
						 * notification sent to other
						 * modules
						 */
	uint32_t        resvd;
} bfa_vf_stats_t;

/**
 * VF attributes returned in queries
 */
typedef struct {
	bfa_vf_state_t  state;		/*!< VF state */
	uint32_t        rsvd;
	wwn_t           fabric_name;	/*!< fabric name */
} bfa_vf_attr_t;

#endif /* __BFA_DEFS_VF_H__ */
