/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

/**
 * @file bfa_fcs_port.h BFA fcs port module public interface
 */

#ifndef __BFA_FCS_PORT_H__
#define __BFA_FCS_PORT_H__

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_port.h>
#include <defs/bfa_defs_pport.h>
#include <defs/bfa_defs_rport.h>
#include <cs/bfa_q.h>
#include <bfa_svc.h>
#include <cs/bfa_wc.h>

struct bfa_fcs_s;
struct bfa_fcs_fabric_s;

/*
* @todo : need to move to a global config file.
 * Maximum Vports supported per physical port or vf.
 */
#define BFA_FCS_MAX_VPORTS_SUPP  255

/*
* @todo : need to move to a global config file.
 * Maximum Rports supported per port (physical/logical).
 */
#define BFA_FCS_MAX_RPORTS_SUPP  256	/* @todo : tentative value */

/**
 * @brief ns data structure
 */
typedef struct {
	bfa_sm_t        sm;	/*!< state machine */
	bfa_timer_t timer;
	struct bfa_fcs_port_s *port;	/*!< parent port */
	bfa_fcxp_t *fcxp;
	bfa_fcxp_wqe_t fcxp_wqe;
} bfa_fcs_port_ns_t;

/**
 * @brief fabric scn data structure
 */
typedef struct {
	bfa_sm_t        sm;	/*!< state machine */
	bfa_timer_t timer;
	struct bfa_fcs_port_s *port;	/*!< parent port */
	bfa_fcxp_t *fcxp;
	bfa_fcxp_wqe_t fcxp_wqe;
} bfa_fcs_port_scn_t;

/**
 * @brief ms fdmi data structure
 */
typedef struct {
	bfa_sm_t        sm;	/*!< state machine */
	bfa_timer_t timer;
	struct bfa_fcs_port_ms_s *ms;	/*!< parent ms */
	bfa_fcxp_t *fcxp;
	bfa_fcxp_wqe_t fcxp_wqe;
	uint8_t         retry_cnt;	/*!< retry count */
	uint8_t	 	   rsvd[3];
} bfa_fcs_port_fdmi_t;

/**
 * @brief fabric ms data structure
 */
typedef struct bfa_fcs_port_ms_s {
	bfa_sm_t        sm;	/*!< state machine */
	bfa_timer_t timer;
	struct bfa_fcs_port_s *port;	/*!< parent port */
	bfa_fcxp_t *fcxp;
	bfa_fcxp_wqe_t fcxp_wqe;
	bfa_fcs_port_fdmi_t fdmi;	/*!< FDMI component of MS */
	uint8_t         retry_cnt;	/*!< retry count */
	uint8_t	 	   rsvd[3];
} bfa_fcs_port_ms_t;

/**
 * @brief fabric topology data structure
 */
typedef struct {
	bfa_fcs_port_ns_t ns;	/*!< NS component of port */
	bfa_fcs_port_scn_t scn;	/*!< scn component of port */
	bfa_fcs_port_ms_t ms;	/*!< MS component of port */
} bfa_fcs_port_fab_t;

/**
 * @brief loop topology data structure
 */

#define 	MAX_ALPA_COUNT 		127

typedef struct {
	uint8_t         num_alpa;	/*!< Num of ALPA entries in the map */
	uint8_t         alpa_pos_map[MAX_ALPA_COUNT];	/*!< ALPA Positional
							 *Map */
	struct bfa_fcs_port_s *port;	/*!< parent port */
} bfa_fcs_port_loop_t;

/**
 * @brief n2n topology data structure
 */

typedef struct {
	uint32_t        rsvd;
	uint16_t        reply_oxid;	/*!< ox_id from the req flogi to be
					 *used in flogi acc */
	wwn_t           rem_port_wwn;	/*!< Attached port's wwn */
} bfa_fcs_port_n2n_t;

/**
 * @brief port topology data structure
 */
typedef union {
	bfa_fcs_port_fab_t pfab;
	bfa_fcs_port_loop_t ploop;
	bfa_fcs_port_n2n_t pn2n;
} bfa_fcs_port_topo_t;

/**
 * @brief port data structure
 */
typedef struct bfa_fcs_port_s {
	bfa_q_t         qe;	/*!< used by port/vport */
	bfa_sm_t        sm;	/*!< logical port SM */
	struct bfa_fcs_fabric_s *fabric;	/*!< parent fabric */
	bfa_port_cfg_t  port_cfg;	/*!< port configuration */
	bfa_timer_t link_timer;	/*!< timer for link offline */
	uint32_t        pid;	/*!< FC address */
	uint16_t        num_rports;	/*!< Num of r-ports */
	bfa_q_t         rport_q;	/*!< queue of discovered r-ports */
	struct bfa_fcs_s *fcs;	/*!< FCS instance */
	bfa_fcs_port_topo_t port_topo;	/*!< fabric/loop/n2n details */
	struct bfad_port_s *bfad_port;	/*!< driver peer instance */
	struct bfa_fcs_vport_s *vport;	/*!< NULL for base ports */
	bfa_fcxp_t *fcxp;
	bfa_fcxp_wqe_t fcxp_wqe;
	bfa_port_stats_t stats;
	bfa_wc_t        wc;	/*!< waiting counter for events */
} bfa_fcs_port_t, bfa_fcs_lport_t;

/**
 * Symbolic Name related defines
 *  Total bytes 255.
 *  Physical Port's symbolic name 128 bytes.
 *  For Vports, Vport's symbolic name is appended to the Physical port's
 *  Symbolic Name.
 *
 *  Physical Port's symbolic name Format : (Total 128 bytes)
 *  Adapter Model number/name : 12 bytes
 *  Driver Version     : 10 bytes
 *  Host Machine Name  : 30 bytes
 * 	Host OS Info	   : 48 bytes
 * 	Host OS PATCH Info : 16 bytes
 *  ( remaining 12 bytes reserved to be used for separator)
 */
#define BFA_FCS_PORT_SYMBNAME_SEPARATOR 		" | "

#define BFA_FCS_PORT_SYMBNAME_MODEL_SZ			12
#define BFA_FCS_PORT_SYMBNAME_VERSION_SZ 		10
#define BFA_FCS_PORT_SYMBNAME_MACHINENAME_SZ 	30
#define BFA_FCS_PORT_SYMBNAME_OSINFO_SZ			48
#define BFA_FCS_PORT_SYMBNAME_OSPATCH_SZ		16

/**
 * Get FC port ID for a logical port.
 */
#define bfa_fcs_port_get_fcid(_lport)	((_lport)->pid)
#define bfa_fcs_port_get_pwwn(_lport)	((_lport)->port_cfg.pwwn)
#define bfa_fcs_port_get_nwwn(_lport)	((_lport)->port_cfg.nwwn)
#define bfa_fcs_port_get_psym_name(_lport)	((_lport)->port_cfg.sym_name)
#define bfa_fcs_port_is_initiator(_lport)	\
			((_lport)->port_cfg.roles & BFA_PORT_ROLE_FCP_IM)
#define bfa_fcs_port_is_target(_lport)	\
			((_lport)->port_cfg.roles & BFA_PORT_ROLE_FCP_TM)
#define bfa_fcs_port_get_nrports(_lport)	\
			((_lport) ? (_lport)->num_rports : 0)

static inline struct bfad_port_s *
bfa_fcs_port_get_drvport(bfa_fcs_port_t *port)
{
	return port->bfad_port;
}

/**
 * @brief Return operating type for the local port.
 */
#define bfa_fcs_port_get_opertype(_lport)	(_lport)->fabric->oper_type

/**
 * @brief Return attached fabric name for the local port.
 */
#define bfa_fcs_port_get_fabric_name(_lport)	(_lport)->fabric->fabric_name

/**
 * @brief Return attached fabric's ip address for the local port.
 */
#define bfa_fcs_port_get_fabric_ipaddr(_lport)	(_lport)->fabric->fabric_ip_addr

/**
 * bfa fcs port public functions
 */
void            bfa_fcs_cfg_base_port(struct bfa_fcs_s *fcs,
				      bfa_port_cfg_t *port_cfg);
bfa_fcs_port_t *bfa_fcs_get_base_port(struct bfa_fcs_s *fcs);
void            bfa_fcs_port_get_rports(bfa_fcs_port_t *port,
					wwn_t rport_wwns[], int *nrports);

wwn_t
bfa_fcs_port_get_rport(bfa_fcs_port_t *port,
					wwn_t wwn,
					int index,
					int nrports,
					bfa_boolean_t bwwn);

bfa_pport_speed_t
bfa_fcs_port_get_rport_max_speed(bfa_fcs_port_t *port);

bfa_fcs_port_t *bfa_fcs_lookup_port(struct bfa_fcs_s *fcs, uint16_t vf_id,
				    wwn_t lpwwn);

void            bfa_fcs_port_get_info(bfa_fcs_port_t *port,
				      bfa_port_info_t *port_info);
void            bfa_fcs_port_get_attr(bfa_fcs_port_t *port,
				      bfa_port_attr_t *port_attr);
void            bfa_fcs_port_get_stats(bfa_fcs_port_t *fcs_port,
				       bfa_port_stats_t *port_stats);
void            bfa_fcs_port_clear_stats(bfa_fcs_port_t *fcs_port);
void            bfa_fcs_port_enable_ipfc_roles(bfa_fcs_port_t *fcs_port);
void            bfa_fcs_port_disable_ipfc_roles(bfa_fcs_port_t *fcs_port);

#endif /* __BFA_FCS_PORT_H__ */
