/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */

#ifndef __BFA_FCPIM_H__
#define __BFA_FCPIM_H__

#include <bfa.h>
#include <bfa_svc.h>
#include <bfi/bfi_fcpim.h>

/*
 * forward declarations
 */
typedef struct bfa_itnim_s bfa_itnim_t;
typedef struct bfa_ioim_s bfa_ioim_t;
typedef struct bfa_tskim_s bfa_tskim_t;
typedef struct bfad_ioim_s bfad_ioim_t;
typedef struct bfad_tskim_s bfad_tskim_t;

/*
 * bfa fcpim module API functions
 */
void		bfa_fcpim_mpio_mode_set(bfa_t *bfa, bfa_boolean_t mpio_mode,
				uint16_t path_tov);
bfa_boolean_t	bfa_fcpim_mpio_mode_get(bfa_t *bfa);
uint16_t	bfa_fcpim_path_tov_get(bfa_t *bfa);
void		bfa_fcpim_qdepth_set(bfa_t *bfa, uint16_t q_depth);
uint16_t	bfa_fcpim_qdepth_get(bfa_t *bfa);

/*
 * bfa itnim API functions
 */
bfa_itnim_t	*bfa_itnim_create(bfa_t *bfa, bfa_rport_t *rport, void *itnim);
void		bfa_itnim_delete(bfa_itnim_t *itnim);
void		bfa_itnim_online(bfa_itnim_t *itnim, bfa_boolean_t seq_rec);
void		bfa_itnim_offline(bfa_itnim_t *itnim);
void		bfa_itnim_get_stats(bfa_itnim_t *itnim,
			bfa_itnim_hal_stats_t *stats);
void		bfa_itnim_clear_stats(bfa_itnim_t *itnim);


/**
 * @brief
 * 		HAL completion callback for bfa_itnim_online().
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void            bfa_cb_itnim_online(void *itnim);

/**
 * @brief
 * 		HAL completion callback for bfa_itnim_offline().
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void            bfa_cb_itnim_offline(void *itnim);
void            bfa_cb_itnim_tov_begin(void *itnim);
void            bfa_cb_itnim_tov(void *itnim);

/**
 * @brief
 * 		HAL notification to FCS/driver for second level error recovery.
 *
 * Atleast one I/O request has timedout and target is unresponsive to
 * repeated abort requests. Second level error recovery should be initiated
 * by starting implicit logout and recovery procedures.
 *
 * @param[in]		itnim		FCS or driver itnim instance
 *
 * return None
 */
void            bfa_cb_itnim_sler(void *itnim);

/*
 * bfa ioim API functions
 */
bfa_ioim_t	*bfa_ioim_alloc(bfa_t *bfa, bfad_ioim_t *dio,
			bfa_itnim_t *itnim, uint16_t nsgles);
void		bfa_ioim_free(bfa_ioim_t *ioim);
void		bfa_ioim_start(bfa_ioim_t *ioim);
void		bfa_ioim_abort(bfa_ioim_t *ioim);
void 		bfa_ioim_delayed_comp(struct bfa_ioim_s *ioim, bfa_boolean_t iotov);


/**
 * @brief
 * 		I/O completion notification.
 *
 * @param[in]		dio			driver IO structure
 * @param[in]		io_status		IO completion status
 * @param[in]		scsi_status		SCSI status returned by target
 * @param[in]		sns_len			SCSI sense length, 0 if none
 * @param[in]		sns_info		SCSI sense data, if any
 * @param[in]		residue			Residual length
 *
 * @return None
 */
void            bfa_cb_ioim_done(void *bfad, bfad_ioim_t *dio,
				  bfi_ioim_status_t io_status,
				  uint8_t scsi_status, int sns_len,
				  uint8_t *sns_info, int32_t residue);

/**
 * @brief
 * 		I/O good completion notification.
 *
 * @param[in]		dio			driver IO structure
 *
 * @return None
 */
void            bfa_cb_ioim_good_comp(void *bfad, bfad_ioim_t *dio);

/**
 * @brief
 * 		I/O abort completion notification
 *
 * @param[in]		dio			driver IO that was aborted
 *
 * @return None
 */
void            bfa_cb_ioim_abort(void *bfad, bfad_ioim_t *dio);
void 			bfa_cb_ioim_resfree(void *hcb_bfad);

/*
 * bfa tskim API functions
 */
bfa_tskim_t	*bfa_tskim_alloc(bfa_t *bfa, bfad_tskim_t *dtsk);
void		bfa_tskim_free(bfa_tskim_t *tskim);
void		bfa_tskim_start(bfa_tskim_t *tskim,
			bfa_itnim_t *itnim, lun_t lun,
			fcp_tm_cmnd_t tm, uint8_t t_secs);
void            bfa_cb_tskim_done(void *bfad, bfad_tskim_t *dtsk,
				   bfi_tskim_status_t tsk_status);

#endif /* __BFA_FCPIM_H__ */

