/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_IPFC_H__
#define __BFA_IPFC_H__

#include <bfa.h>

typedef struct bfad_pkt_s bfad_pkt_t;
typedef struct bfa_ipfc_pkt_s bfa_ipfc_pkt_t;

/**
 * Packet receive status
 */
typedef enum {
	BFA_IPFC_STATUS_OK = 0,		/*!< good packet		 */
	BFA_IPFC_STATUS_PARTIAL = 1,	/*!< only partial frame is recvd */
	BFA_IPFC_STATUS_ERROR = 2,	/*!< driver requested return	 */
} bfa_ipfc_status_t;

/*
 * bfa ipfc API functions
 */
void            bfa_ipfc_meminfo(int npkts, bfa_meminfo_t *meminfo);
void            bfa_ipfc_attach(bfa_t *bfa, void *dipfc, int npkts,
				    bfa_meminfo_t *meminfo);
void            bfa_ipfc_detach(bfa_t *bfa);
bfa_ipfc_pkt_t  *bfa_ipfc_postpkt(bfa_t *bfa, bfad_pkt_t *dpkt);
void            bfa_ipfc_flushpkt(bfa_ipfc_pkt_t *pkt);

/**
 * @brief
 * 		Completion callback for bfa_ipfc_detach().
 *
 * @param[in]		dipfc		driver IPFC instance
 *
 * @return None
 */
void            bfa_cb_ipfc_detach(void *dipfc);

/**
 * @brief
 * 		Callback to process a received packet.
 *
 * @param[in]		dipfc		driver IPFC instance
 * @param[in]		dpkt		driver packet
 * @param[in]		status		Status of received packet.
 * @param[in]		pktlen		Length of packet received.
 * @param[in]		residue		Residual length in bytes of packet
 *					that was not copied due to lack of
 *					space.
 *
 * @return None
 */
typedef
void            (*send_cbfn_t)(void *dipfc, bfad_pkt_t *dpkt,
				     bfa_ipfc_status_t status,
				     uint32_t pktlen, uint32_t residue);

void		bfa_hcb_ipfc_pktrecv(void *dipfc, bfad_pkt_t *dpkt,
				     bfa_ipfc_status_t status,
				     uint32_t pktlen, uint32_t residue);


/**
 * @brief
 * 		Get number of SG elements required to map receive packet buffer.
 *
 * @param[in]		dpkt		driver packet
 *
 * @return Number of SG elements for the packet.
 */
int             bfa_hcb_ipfc_nsgles(bfad_pkt_t *dpkt);

/**
 * @brief
 * 		Get total packet buffer size in bytes.
 *
 * @param[in]		dpkt		driver packet
 *
 * @return Packet size in bytes.
 */
uint16_t             bfa_hcb_ipfc_pktlen(bfad_pkt_t *dpkt);


/**
 * @brief
 * 		Get SG element of the mapped packet buffer given the SG index.
 *
 * @param[in]		dpkt		driver packet
 * @param[in]		sge_idx		SG element index
 * @param[out]		sge		SG element contents
 *
 * @return None
 */
void            bfa_hcb_ipfc_sge(bfad_pkt_t *dpkt, int sge_idx,
				 bfa_sge_t *sge);

#endif /* __BFA_IPFC_H__ */
