/*
 * Copyright (c)  2005-2008 Brocade Communications Systems, Inc.
 * All rights reserved
 * www.brocade.com
 *
 * See LICENSE.bfa for copyright and licensing details.
 */
#ifndef __BFA_SVC_H__
#define __BFA_SVC_H__

/*
 * forward declarations
 */
typedef struct bfa_fcxp_s bfa_fcxp_t;

#include <defs/bfa_defs_status.h>
#include <defs/bfa_defs_pport.h>
#include <defs/bfa_defs_rport.h>
#include <defs/bfa_defs_qos.h>
#include <cs/bfa_sm.h>
#include <bfa.h>

/**
 * @brief
 * 		BFA rport information.
 */
typedef struct {
	uint16_t        max_frmsz;	/*!< max rcv pdu size               */
	uint32_t        pid:24,		/*!< remote port ID                 */
				    rsvd1:8;
	uint32_t        local_pid:24,	/*!< local port ID                  */
		           	cisc:8;		/*!< CIRO supported		    */
	uint8_t         fc_class;	/*!< supported FC classes. fc_cos_t */
	uint8_t         vf_en;		/*!< virtual fabric enable          */
	uint16_t        vf_id;		/*!< virtual fabric ID              */
	bfa_pport_speed_t speed;    /* Rport's current speed.  Default 1G */
} bfa_rport_info_t;

/**
 * BFA rport data structure
 */
typedef struct {
	bfa_q_t               qe;	 /*!< queue element                 */
	bfa_sm_t              sm;	 /*!< state machine                 */
	bfa_t                 *bfa;	 /*!< backpointer to BFA            */
	void                  *rport_drv;/*!< fcs/driver rport object       */
	uint16_t              fw_handle; /*!< firmware rport handle         */
	uint16_t              rport_tag; /*!< BFA rport tag                 */
	bfa_rport_info_t      rport_info;/*!< rport info from *fcs/driver   */
	struct bfa_reqq_wait_s reqq_wait; /*!< to wait for room in reqq      */
	bfa_cb_qe_t          hcb_qe;	 /*!< BFA callback qelem            */
	bfa_rport_hal_stats_t stats;	 /*!< BFA rport statistics          */
	bfa_rport_qos_attr_t  qos_attr;  
	union a {
		bfa_status_t    status;	 /*!< f/w status                    */
		void            *fw_msg; /*!< QoS scn event                 */
	} event_arg;
} bfa_rport_t;
#define BFA_RPORT_FC_COS(_rport)	((_rport)->rport_info.fc_class)

/**
 * Send completion callback.
 */
typedef void    (*bfa_cb_fcxp_send_t) (void *bfad_fcxp, bfa_fcxp_t *fcxp,
					void *cbarg, bfa_status_t req_status,
					uint32_t rsp_len, uint32_t resid_len,
					fchs_t *rsp_fchs);

/**
 * BFA fcxp allocation (asynchronous)
 */
typedef void    (*bfa_fcxp_alloc_cbfn_t) (void *cbarg, bfa_fcxp_t *fcxp);
typedef struct {
	bfa_q_t         qe;
	bfa_fcxp_alloc_cbfn_t alloc_cbfn;
	void           *alloc_cbarg;
} bfa_fcxp_wqe_t;

typedef         uint64_t(*bfa_fcxp_get_sgaddr_t) (void *bfad_fcxp, int sgeid);
typedef         uint32_t(*bfa_fcxp_get_sglen_t) (void *bfad_fcxp, int sgeid);

#define BFA_UF_BUFSZ	(2 * 1024 + 256)

/**
 * @todo private
 */
typedef struct {
	uint8_t         d[BFA_UF_BUFSZ];
} bfa_uf_buf_t;

/**
 * @brief
 */
typedef struct bfa_uf_s {
	bfa_q_t         qe;		/*!< queue element		  */
	bfa_t      	*bfa;		/*!< bfa instance		  */
	uint16_t        uf_tag;		/*!< identifying tag f/w messages */
	uint16_t        vf_id;
	uint16_t        src_rport_handle;
	uint16_t        rsvd;
	uint8_t        	*data_ptr;
	uint16_t        data_len;	/*!< actual receive length	  */
	uint16_t        pb_len;		/*!< posted buffer length	  */
	void           	*buf_kva;	/*!< buffer virtual address	  */
	uint64_t        buf_pa;		/*!< buffer physical address	  */
	bfa_cb_qe_t    hcb_qe;		/*!< comp: BFA comp qelem	  */
	bfa_sge_t   	sges[BFI_SGE_INLINE_MAX];
} bfa_uf_t;

typedef void    (*bfa_cb_pport_t) (void *cbarg, bfa_status_t status);

/*
 * bfa pport API functions
 */
bfa_status_t    bfa_pport_enable(bfa_t *bfa);
bfa_status_t    bfa_pport_disable(bfa_t *bfa);
bfa_status_t    bfa_pport_cfg_speed(bfa_t *bfa,
					bfa_pport_speed_t speed);
bfa_pport_speed_t bfa_pport_get_speed(bfa_t *bfa);
bfa_status_t    bfa_pport_cfg_topology(bfa_t *bfa,
					   bfa_pport_topology_t topo);
bfa_pport_topology_t bfa_pport_get_topology(bfa_t *bfa);
bfa_status_t    bfa_pport_cfg_hardalpa(bfa_t *bfa, uint8_t alpa);
bfa_boolean_t   bfa_pport_get_hardalpa(bfa_t *bfa, uint8_t *alpa);
uint8_t         bfa_pport_get_myalpa(bfa_t *bfa);
bfa_status_t    bfa_pport_clr_hardalpa(bfa_t *bfa);
bfa_status_t    bfa_pport_cfg_maxfrsize(bfa_t *bfa, uint16_t maxsize);
uint16_t        bfa_pport_get_maxfrsize(bfa_t *bfa);
uint8_t         bfa_pport_get_rx_bbcredit(bfa_t *bfa);
bfa_status_t    bfa_pport_trunk_enable(bfa_t *bfa, uint8_t bitmap);
bfa_status_t    bfa_pport_trunk_disable(bfa_t *bfa);
bfa_boolean_t   bfa_pport_trunk_query(bfa_t *bfa, uint32_t *bitmap);
void            bfa_pport_get_attr(bfa_t *bfa,
				       bfa_pport_attr_t *attr);

wwn_t			bfa_pport_get_wwn(bfa_t *bfa, bfa_boolean_t node);

bfa_status_t    bfa_pport_get_stats(bfa_t *bfa,
					bfa_pport_stats_t *stats,
					bfa_cb_pport_t cbfn, void *cbarg);
bfa_status_t    bfa_pport_clear_stats(bfa_t *bfa, bfa_cb_pport_t cbfn,
					  void *cbarg);
void            bfa_pport_event_register(bfa_t *bfa,
					     void (*event_cbfn) (void *cbarg,
							bfa_pport_event_t
							event),
					     void *event_cbarg);
bfa_boolean_t   bfa_pport_is_disabled(bfa_t *bfa);
void            bfa_pport_cfg_qos(bfa_t *bfa, bfa_boolean_t on_off);
void            bfa_pport_cfg_ratelim(bfa_t *bfa,
					  bfa_boolean_t on_off);
bfa_status_t    bfa_pport_cfg_ratelim_speed(bfa_t *bfa, bfa_pport_speed_t speed);
bfa_pport_speed_t bfa_pport_get_ratelim_speed(bfa_t *bfa);
bfa_boolean_t     bfa_pport_is_ratelim(bfa_t *bfa);

void            bfa_pport_set_tx_bbcredit(bfa_t *bfa,
					      uint16_t tx_bbcredit);
void            bfa_pport_busy(bfa_t *bfa, bfa_boolean_t status);
void            bfa_pport_beacon(bfa_t *bfa, bfa_boolean_t beacon,
					bfa_boolean_t link_e2e_beacon);
void            bfa_cb_pport_event(void *cbarg, bfa_pport_event_t event);
void            bfa_pport_qos_get_attr(bfa_t *bfa, bfa_qos_attr_t *qos_attr);
void		    bfa_pport_qos_get_vc_attr(bfa_t *bfa, bfa_qos_vc_attr_t *qos_vc_attr);
bfa_status_t    bfa_pport_get_qos_stats(bfa_t *bfa, 
				bfa_pport_qos_stats_t *stats,
				bfa_cb_pport_t cbfn, void *cbarg);
bfa_status_t    bfa_pport_clear_qos_stats(bfa_t *bfa, bfa_cb_pport_t cbfn, 
                void *cbarg);


/*
 * bfa rport API functions
 */
bfa_rport_t	*bfa_rport_create(bfa_t *bfa, void *rport_drv);
void		bfa_rport_delete(bfa_rport_t *rport);
void        bfa_rport_online(bfa_rport_t *rport,
				     bfa_rport_info_t *rport_info);
void        bfa_rport_offline(bfa_rport_t *rport);
void 		bfa_rport_speed(bfa_rport_t *rport, bfa_pport_speed_t speed);
void		bfa_rport_get_stats(bfa_rport_t *rport,
					bfa_rport_hal_stats_t *stats);
void		bfa_rport_clear_stats(bfa_rport_t *rport);
void        bfa_cb_rport_online(void *rport);
void        bfa_cb_rport_offline(void *rport);
void        bfa_cb_rport_qos_scn_flowid(void *rport, 
					bfa_rport_qos_attr_t old_qos_attr,
					bfa_rport_qos_attr_t new_qos_attr);
void        bfa_cb_rport_qos_scn_prio(void *rport,
				      bfa_rport_qos_attr_t old_qos_attr,
				      bfa_rport_qos_attr_t new_qos_attr);
void		bfa_rport_get_qos_attr(bfa_rport_t *rport,
					bfa_rport_qos_attr_t *qos_attr);

/*
 * bfa fcxp API functions
 */
bfa_fcxp_t	*bfa_fcxp_alloc(void *bfad_fcxp, bfa_t *bfa,
				   int nreq_sgles, int nrsp_sgles,
				   bfa_fcxp_get_sgaddr_t get_req_sga,
				   bfa_fcxp_get_sglen_t get_req_sglen,
				   bfa_fcxp_get_sgaddr_t get_rsp_sga,
				   bfa_fcxp_get_sglen_t get_rsp_sglen);
void            bfa_fcxp_alloc_wait(bfa_t *bfa,
					bfa_fcxp_wqe_t *wqe,
					bfa_fcxp_alloc_cbfn_t alloc_cbfn,
					void *cbarg);
void            bfa_fcxp_walloc_cancel(bfa_t *bfa,
					   bfa_fcxp_wqe_t *wqe);
void            bfa_fcxp_discard(bfa_fcxp_t *fcxp);

void           *bfa_fcxp_get_reqbuf(bfa_fcxp_t *fcxp);
uint32_t        bfa_fcxp_get_reqbufsz(bfa_fcxp_t *fcxp);
uint32_t	bfa_fcxp_get_maxrsp(bfa_t *bfa);
void           *bfa_fcxp_get_rspbuf(bfa_fcxp_t *fcxp);

void            bfa_fcxp_free(bfa_fcxp_t *fcxp);
void            bfa_fcxp_send(bfa_fcxp_t *fcxp,
				  bfa_rport_t *rport, uint16_t vf_id,
				  bfa_boolean_t cts, fc_cos_t cos,
				  uint32_t reqlen, fchs_t *fchs,
				  bfa_cb_fcxp_send_t cbfn, void *cbarg,
				  uint32_t rsp_maxlen, uint8_t rsp_timeout);
bfa_status_t    bfa_fcxp_abort(bfa_fcxp_t *fcxp);

static inline void *
bfa_uf_get_frmbuf(bfa_uf_t *uf)
{
	return uf->data_ptr;
}

static inline   uint16_t
bfa_uf_get_frmlen(bfa_uf_t *uf)
{
	return uf->data_len;
}

/**
 * @brief
 *	Callback prototype for unsolicited frame receive handler.
 *
 * @param[in]		cbarg		callback arg for receive handler
 * @param[in]		uf		unsolicited frame descriptor
 *
 * @return None
 */
typedef void    (*bfa_cb_uf_recv_t) (void *cbarg, bfa_uf_t *uf);

/*
 * bfa uf API functions
 */
void		bfa_uf_recv_register(bfa_t *bfa, bfa_cb_uf_recv_t ufrecv,
				void *cbarg);
void		bfa_uf_free(bfa_uf_t *uf);

#endif /* __BFA_SVC_H__ */

